searchData={"items":[{"type":"task","title":"mix api.gen","doc":"Generate code from an Open API description\n\nThis task requires two arguments:\n\n* The configuration profile to use, and\n* The location of the OpenAPI description.\n\nThe description location can be given as an absolute path or a path relative to the Mix project\nwhere this task is run.","ref":"Mix.Tasks.Api.Gen.html"},{"type":"task","title":"Example - mix api.gen","doc":"```shell\nmix api.gen default ../rest-api-description/spec.yaml\n```","ref":"Mix.Tasks.Api.Gen.html#module-example"},{"type":"module","title":"OpenAPI","doc":"Code generator for OpenAPI REST API descriptions\n\n> #### Note {:.info}\n>\n> It is not expected that clients will call functions in this module directly. Instead, consider\n> using the `mix api.gen` task.\n\nThis module provides a single function, `run/2`, that accepts the name of a configuration\nprofile and a list of files containing JSON or Yaml OpenAPI descriptions. It runs the three\nphases of code generation:\n\n  * `OpenAPI.Reader`\n  * `OpenAPI.Processor`\n  * `OpenAPI.Renderer`\n\nAt the end, an `OpenAPI.State` struct is returned with all of the data that was used to\ngenerate the client code.\n\nFor more information, see the [readme](/README.md) or the available guides on\n[configuration](/guides/configuration.md), [plugins](/guides/plugins.md), or\n[creating a client library](/guides/client-author-guide.md).","ref":"OpenAPI.html"},{"type":"function","title":"OpenAPI.run/2","doc":"Using the given configuration `profile`, read the OpenAPI description contained in `files` and\ngenerate client code","ref":"OpenAPI.html#run/2"},{"type":"module","title":"OpenAPI.Call","doc":"Information about the original invocation of the code generator\n\n> #### Note {:.info}\n>\n> It is not expected that clients will interact with this module directly.","ref":"OpenAPI.Call.html"},{"type":"type","title":"OpenAPI.Call.t/0","doc":"","ref":"OpenAPI.Call.html#t:t/0"},{"type":"module","title":"OpenAPI.State","doc":"State for the overall code generation\n\nThis struct contains data that is passed between phases of the generation. It includes the\nfollowing fields:\n\n  * `call`: Information about the original call to the generator, including the configuration\n    profile and the spec files passed.\n  * `files`: After the render phase, contains a list of `t:OpenAPI.Renderer.File.t/0` structs\n    that were written.\n  * `operations`: After the process phase, contains a list of `t:OpenAPI.Processor.Operation.t/0`\n    structs that were processed.\n  * `schemas`: After the process phase, contains a map of schema references to their\n    `t:OpenAPI.Processor.Schema.t/0` structs. The reference-keyed map is included to make it\n    easier for render callbacks to look up a schema by its reference, as this is often how\n    operations will refer to them.\n  * `schema_specs_by_path`: After the read phase, contains a map of all schema specs keyed on\n    their base file and last referenced paths. This allows for easy lookup of schema specs from\n    `t:OpenAPI.Spec.ref/0` references.\n  * `spec`: After the read phase, contains the parsed and merged OpenAPI description(s).\n\nAll of this state is managed by the code generator between phases, and it is unlikely that a\ncallback would need to read or write to this struct directly.","ref":"OpenAPI.State.html"},{"type":"type","title":"OpenAPI.State.t/0","doc":"OpenAPI generator state","ref":"OpenAPI.State.html#t:t/0"},{"type":"type","title":"OpenAPI.State.yaml/0","doc":"Raw Yaml input.","ref":"OpenAPI.State.html#t:yaml/0"},{"type":"module","title":"OpenAPI.Reader","doc":"Phase one of code generation\n\nThe **read** phase begins with one or more filenames (JSON or YAML) to read. Each document\nshould be the root of an OpenAPI description (usually beginning with an `openapi` key and\nversion number). As the first step, they are parsed into large, nested maps and stored in the\nstate based on their filename. Then, each document is parsed into structs based on the OpenAPI\ndocument definition.\n\nNote that this library is not meant to \"validate\" OpenAPI descriptions. Errors may be thrown\nonly when there are details missing that affect the operation of the code generator. Because of\nthis, you may be able to \"get away with\" generating code from an incomplete or invalid document.\n\n> #### Note {:.info}\n>\n> Functions in this module are used by the main `OpenAPI` module. It is unlikely that you will\n> call these functions directly, unless you are reimplementing one of the core functions. If\n> this happens, please share your use-case with the maintainers; a plugin might be warranted.","ref":"OpenAPI.Reader.html"},{"type":"module","title":"Configuration - OpenAPI.Reader","doc":"All configuration for this phase lives under the `reader` key in each profile:\n\n    config :oapi_generator,\n      default: [\n        reader: [\n          ...\n        ]\n      ]\n\nThe following options are available:\n\n  * `additional_files`: List of paths to supplemental root files of an API description. See\n    **Supplemental Descriptions** below for more information. Defaults to no additional files.\n\n  * `file`: Path to the (likely vendor-supplied) root file of an API description. This\n    information is often supplied to the `mix api.gen` task, since individual client library\n    maintainers may keep the API description document in different locations. However, if the\n    API description is included in the client library repository, then this configuration can be\n    used instead. If unset, the filename must be supplied to `mix api.gen`.","ref":"OpenAPI.Reader.html#module-configuration"},{"type":"module","title":"Supplemental Descriptions - OpenAPI.Reader","doc":"Sometimes, the OpenAPI description provided by a vendor is incomplete. Client library authors\nmay choose to write supplemental descriptions that \"fill in the gaps\". Although these files\nmust contain the root of an OpenAPI description (including an `openapi` key and version number)\nthey may reference parts of the vendor-provided specification.\n\nFor example, GitHub provides special ID-based API routes like `/repositories/{id}` that are\nnot documented in its OpenAPI description. A client library author may create a supplemental\ndescription that looks like this (abbreviated):\n\n```yaml\nopenapi: 3.1.0\npaths:\n  \"/repositories/{id}\":\n    get:\n      summary: Get a repository by ID\n      operationId: repos/get-by-id\n      parameters:\n      - \"$ref\": \"#/components/parameters/id\"\n      responses:\n        '200':\n          description: Response\n          content:\n            application/json:\n              schema:\n                \"$ref\": \"#/components/schemas/full-repository\"\n```\n\nNote that references to schemas contained in the vendor-supplied description must be modified\nto use accurate paths. Assuming the supplemental file is committed in the client library\nrepository, authors may use the `additional_files` configuration to ensure it is always\nprocessed along with the vendor-supplied description:\n\n    config :oapi_generator,\n      default: [\n        reader: [\n          additional_files: [\"path/to/supplement.yaml\"]\n        ]\n      ]\n\nDuring the reading phase, the two root documents will be combined as if each root key were\nconcatenated (for example, the paths from one file were copied to the other). Keys like `info`\nwill always use the vendor-supplied data. After the reading phase, the documents will be treated\nas one API description, although plugins will have the opportunity to see the source of the data\nwhen necessary.","ref":"OpenAPI.Reader.html#module-supplemental-descriptions"},{"type":"function","title":"OpenAPI.Reader.ensure_file/2","doc":"","ref":"OpenAPI.Reader.html#ensure_file/2"},{"type":"function","title":"OpenAPI.Reader.run/1","doc":"Run the reading phase of the code generator","ref":"OpenAPI.Reader.html#run/1"},{"type":"module","title":"OpenAPI.Reader.Config","doc":"Configuration for the read phase\n\nFor more information, see the **Configuration** section of the documentation for\n`OpenAPI.Reader`.","ref":"OpenAPI.Reader.Config.html"},{"type":"type","title":"OpenAPI.Reader.Config.t/0","doc":"","ref":"OpenAPI.Reader.Config.html#t:t/0"},{"type":"module","title":"OpenAPI.Reader.State","doc":"State of the reader phase of code generation\n\nThis struct is created at the beginning of the read phase using data from the overall\n`OpenAPI.State`. It is fully managed by the read phase, and it is unlikely that client libraries\nwould read or write to this struct.","ref":"OpenAPI.Reader.State.html"},{"type":"type","title":"OpenAPI.Reader.State.decoder/0","doc":"Decode function for raw Yaml","ref":"OpenAPI.Reader.State.html#t:decoder/0"},{"type":"type","title":"OpenAPI.Reader.State.decoder/1","doc":"Decode function for raw Yaml","ref":"OpenAPI.Reader.State.html#t:decoder/1"},{"type":"type","title":"OpenAPI.Reader.State.t/0","doc":"OpenAPI reader state","ref":"OpenAPI.Reader.State.html#t:t/0"},{"type":"type","title":"OpenAPI.Reader.State.yaml/0","doc":"Raw Yaml input","ref":"OpenAPI.Reader.State.html#t:yaml/0"},{"type":"behaviour","title":"OpenAPI.Processor","doc":"Phase two of code generation\n\nThe **process** phase begins with a decoded API description from the **read** phase. It may\nrepresent the contents of one or more files, including one or more root descriptions if\nsupplemental files were used.\n\nThis library takes an \"operation first\" mindset when processing the description. Schemas are\nignored until they are referenced by an operation (ex. as a response body). It is the job of\nthis phase to observe all of the operations and their referenced schemas, process them into\ndata structures more relevant to code generation, and prepare the data for rendering.","ref":"OpenAPI.Processor.html"},{"type":"behaviour","title":"Customization - OpenAPI.Processor","doc":"At several points during code generation, it may be useful to customize the behaviour of the\nprocessor. For this purpose, this module is a Behaviour with most of its critical logic\nimplemented as optional callbacks.","ref":"OpenAPI.Processor.html#module-customization"},{"type":"callback","title":"OpenAPI.Processor.ignore_operation?/2","doc":"Whether to render the given operation in the generated code\n\nIf this function returns `true`, the operation will not appear in the generated code.\n\nSee `OpenAPI.Processor.Ignore.ignore_operation?/2` for the default implementation.","ref":"OpenAPI.Processor.html#c:ignore_operation?/2"},{"type":"callback","title":"OpenAPI.Processor.ignore_schema?/2","doc":"Whether to render the given schema in the generated code\n\nIf this function returns `true`, the schema will not appear in the generated code (unless it\nreturns `false` when presented in another context) and a plain `map` will be used as its type.\n\nSee `OpenAPI.Processor.Ignore.ignore_schema?/2` for the default implementation.","ref":"OpenAPI.Processor.html#c:ignore_schema?/2"},{"type":"callback","title":"OpenAPI.Processor.operation_docstring/3","doc":"Construct a docstring for the given operation\n\nThis function accepts the operation spec as well as a list of the **processed** query params\nassociated with the operation.\n\nSee `OpenAPI.Processor.Operation.docstring/3` for the default implementation.","ref":"OpenAPI.Processor.html#c:operation_docstring/3"},{"type":"callback","title":"OpenAPI.Processor.operation_function_name/2","doc":"Choose the name of the client function for the given operation\n\nThis function accepts the operation spec and chooses a name for the client function that will\nbe generated. The name must be unique within its module (see `c:operation_module_names/2`).\n\nSee `OpenAPI.Processor.Naming.operation_function/2` for the default implementation.","ref":"OpenAPI.Processor.html#c:operation_function_name/2"},{"type":"callback","title":"OpenAPI.Processor.operation_module_names/2","doc":"Choose the names of the client function modules for the given operation\n\nEach operation may have multiple client functions in different modules. This function accepts\nthe operation spec and returns a list of all of its operation modules.\n\nSee `OpenAPI.Processor.Naming.operation_modules/2` for the default implementation.","ref":"OpenAPI.Processor.html#c:operation_module_names/2"},{"type":"callback","title":"OpenAPI.Processor.operation_request_body/2","doc":"Collect a list of request content types and their associated schemas\n\nThis function accepts the operation spec and returns a list of tuples containing the content\ntype (ex. \"application/json\") and the schema associated with that type.\n\nSee `OpenAPI.Processor.Operation.request_body/2` for the default implementation.","ref":"OpenAPI.Processor.html#c:operation_request_body/2"},{"type":"callback","title":"OpenAPI.Processor.operation_request_method/2","doc":"Choose and cast the request method for the given operation\n\nThis function accepts the operation spec and must return the (lowercase) atom representing the\nHTTP method.\n\nSee `OpenAPI.Processor.Operation.request_method/2` for the default implementation.","ref":"OpenAPI.Processor.html#c:operation_request_method/2"},{"type":"callback","title":"OpenAPI.Processor.operation_response_body/2","doc":"Collect a list of response status codes and their associated schemas\n\nThis function accepts the operation spec and returns a list of tuples containing the status\ncodes (ex. `200` or `:default`) and a list of tuples with the possible content types and the\nschema associated with that code and type.\n\nSee `OpenAPI.Processor.Operation.response_body/2` for the default implementation.","ref":"OpenAPI.Processor.html#c:operation_response_body/2"},{"type":"function","title":"OpenAPI.Processor.run/1","doc":"Run the processing phase of the code generator\n\nThis functions is used by the main `OpenAPI` module. It is unlikely that you will call this\nfunction directly, unless you are reimplementing one of the core functions. If this happens,\nplease share your use-case with the maintainers; a plugin might be warranted.","ref":"OpenAPI.Processor.html#run/1"},{"type":"callback","title":"OpenAPI.Processor.schema_format/2","doc":"Choose the output format for the given schema\n\nSchemas can be output in a number of different ways, the most common of which are structs and\ntyped maps. This callback chooses which output format to use for a processed schema. This\ndecision may influence the naming (module and type) given to the schema.\n\nThe special value `:unknown` may be returned from this callback to delay / re-enqueue the\nschema for processing. This may appropriate, for example, if the schema's parent has not yet\nbeen processed. Note that continually returning `:unknown` can cause an infinite loop.\n\nSee `OpenAPI.Processor.Format.schema_format/2` for the default implementation.","ref":"OpenAPI.Processor.html#c:schema_format/2"},{"type":"callback","title":"OpenAPI.Processor.schema_module_and_type/2","doc":"Choose the name of the module and type for the given schema\n\nEach module may contain multiple schemas with distinct types. By convention, the default\ntype (used for single-schema modules) is `:t`, as in `MySchema.t()`. Additional schemas in the\nsame module may represent variations on the default type, such as a `User.public()` schema\ncontaining a reduced set of fields from `User.t()`.\n\nSee `OpenAPI.Processor.Naming.schema_module_and_type/2` for the default implementation.","ref":"OpenAPI.Processor.html#c:schema_module_and_type/2"},{"type":"module","title":"OpenAPI.Processor.Format","doc":"Default implementation for format-related callbacks\n\nThis module contains the default implementations for:\n\n  * `c:OpenAPI.Processor.schema_format/2`","ref":"OpenAPI.Processor.Format.html"},{"type":"module","title":"Configuration - OpenAPI.Processor.Format","doc":"This implementation does not currently use any configuration.","ref":"OpenAPI.Processor.Format.html#module-configuration"},{"type":"function","title":"OpenAPI.Processor.Format.schema_format/2","doc":"","ref":"OpenAPI.Processor.Format.html#schema_format/2"},{"type":"type","title":"OpenAPI.Processor.Format.format/0","doc":"","ref":"OpenAPI.Processor.Format.html#t:format/0"},{"type":"module","title":"OpenAPI.Processor.Ignore","doc":"Ignore operations and schemas from the description\n\nThis module contains the default implementations for:\n\n  * `c:OpenAPI.Processor.ignore_operation?/2`\n  * `c:OpenAPI.Processor.ignore_schema?/2`","ref":"OpenAPI.Processor.Ignore.html"},{"type":"module","title":"Configuration - OpenAPI.Processor.Ignore","doc":"All configuration for the functions in this module is contained in a key `ignore` of the active\nconfiguration profile. For example:\n\n    # config/config.exs\n\n    config :oapi_generator, default: [\n      ignore: [\n        \"IgnoredOperation\",\n        ~r\"/components/schemas/ignored-\"\n      ]\n    ]\n\nEach element of the `ignored` list is a pattern. Patterns are compared against the operation IDs\nand paths of an operation, and the paths and titles of a schema. If a string is given, it is\ncompared for equality. Regular expressions are tested using `Regex.match?/2`.\n\nIf any pattern matches the tested operation or schema, it will be excluded.","ref":"OpenAPI.Processor.Ignore.html#module-configuration"},{"type":"function","title":"OpenAPI.Processor.Ignore.ignore_operation?/2","doc":"Ignore operations based on configured patterns of IDs and paths\n\nDefault implementation of `c:OpenAPI.Processor.ignore_operation?/2`.\n\nIn this implementation, patterns from the `ignore` configuration key are compared against each\noperation ID and path. If a string pattern matches exactly, or regular expression pattern\nmatches according to `Regex.match?/2`, the operation will be ignored.","ref":"OpenAPI.Processor.Ignore.html#ignore_operation?/2"},{"type":"function","title":"OpenAPI.Processor.Ignore.ignore_schema?/2","doc":"Ignore schemas based on configured patterns of paths and titles\n\nDefault implementation of `c:OpenAPI.Processor.ignore_schema?/2`.\n\nIn this implementation, patterns from the `ignore` configuration key are compared against each\nschema title and path. If a string pattern matches exactly, or regular expression pattern\nmatches according to `Regex.match?/2`, the schema will be ignored. This often means the type\nwill be replaced by a plain `map`.","ref":"OpenAPI.Processor.Ignore.html#ignore_schema?/2"},{"type":"type","title":"OpenAPI.Processor.Ignore.definition/0","doc":"Definition of a module to ignore","ref":"OpenAPI.Processor.Ignore.html#t:definition/0"},{"type":"module","title":"OpenAPI.Processor.Naming","doc":"Default implementation for naming-related callbacks\n\nThis module contains the default implementations for:\n\n  * `c:OpenAPI.Processor.operation_function_name/2`\n  * `c:OpenAPI.Processor.operation_module_names/2`\n  * `c:OpenAPI.Processor.schema_module_and_type/2`\n\nIt also includes several helper functions that are used by the default implementations. Library\nauthors implementing their own naming-related callbacks may find these helpful.","ref":"OpenAPI.Processor.Naming.html"},{"type":"module","title":"Configuration - OpenAPI.Processor.Naming","doc":"All configuration offered by the functions in this module lives under the `naming` key of the\nactive configuration profile. For example (default values shown):\n\n    # config/config.exs\n\n    config :oapi_generator, default: [\n      naming: [\n        default_operation_module: Operations,\n        group: [],\n        merge: [],\n        rename: []\n      ]\n    ]","ref":"OpenAPI.Processor.Naming.html#module-configuration"},{"type":"function","title":"OpenAPI.Processor.Naming.group_schema/2","doc":"Group schema modules into configured namespaces\n\nThis function accepts a tuple with the module and type of a schema as strings, along with the\nprocessor state, and returns a modified tuple according to the configured groups.","ref":"OpenAPI.Processor.Naming.html#group_schema/2"},{"type":"function","title":"Discussion - OpenAPI.Processor.Naming.group_schema/2","doc":"Schemas in an OpenAPI description can have extensively long names. For example, GitHub has a\nschema called `actions-cache-usage-by-repository`. Along with all other actions-related schemas,\nwe can cut down the top-level module namespace by grouping on `Actions` or even further:\n\n    group: [\n      Actions,\n      Actions.CacheUsage\n    ]\n\nEven simple renaming and groups can take a raw OpenAPI description and turn it into a library\nthat feels friendly to users.","ref":"OpenAPI.Processor.Naming.html#group_schema/2-discussion"},{"type":"function","title":"Configuration - OpenAPI.Processor.Naming.group_schema/2","doc":"Module namespaces can be configured as a list of modules in the `naming.group` key of a\nconfiguration profile:\n\n    config :oapi_generator, default: [\n      naming: [\n        group: [\n          Author,\n          Author.Bio\n          Comment,\n          # ...\n        ]\n      ]\n    ]","ref":"OpenAPI.Processor.Naming.html#group_schema/2-configuration"},{"type":"function","title":"Examples - OpenAPI.Processor.Naming.group_schema/2","doc":"The configuration above includes three module namespaces for grouping: `Author`, `Author.Bio`,\nand `Comment`. These rules would create the following transformations (types omitted because\nthey do not change):\n\n    AuthorAvatar    => Author.Avatar\n    AuthorBio       => Author.Bio\n    AuthorBioUpdate => Author.Bio.Update\n    PostComment     => PostComment\n\nNote that the desired grouping must appear at the start of the module name: `PostComment` is\nunaffected by the `Comment` group configuration. As a result, it is also important that `Author`\nappear in the configuration before `Author.Bio`, otherwise `Author.Bio` would fail to match the\nbeginning of `AuthorBioUpdate` resulting in `Author.BioUpdate` (since the `Author` configuration\nwould still match afterwards).","ref":"OpenAPI.Processor.Naming.html#group_schema/2-examples"},{"type":"function","title":"OpenAPI.Processor.Naming.merge_schema/2","doc":"Merge schemas based on configured pairs of patterns and replacements\n\nThis function accepts a tuple with the module and type of a schema as strings, along with the\nprocessor state, and returns a modified tuple according to the configured merges.","ref":"OpenAPI.Processor.Naming.html#merge_schema/2"},{"type":"function","title":"Discussion - OpenAPI.Processor.Naming.merge_schema/2","doc":"OpenAPI descriptions may have multiple schemas that are closely related or even duplicated.\nMerging gives the power to consolidate these schemas into a single struct that is easy to use.\n\nFor example, the GitHub API description used to have schemas `repository`, `full-repository`,\nand `nullable-repository`. While the \"full\" repository added additional properties, the\n\"nullable\" variant was just that: all of the same properties, but the schema was nullable. This\nkind of oddity in the OpenAPI specification is exactly what makes most generated code difficult\nto use.\n\nThe following merge settings would help clean this up:\n\n    merge: [\n      {\"FullRepository\", \"Repository\"},\n      {~r/^Nullable/, \"\"}\n    ]\n\nIn the first line, we tell the generator to merge `FullRepository` into `Repository` (the\noriginal module names based on the names of the schemas). Because the destination module appears\nat the end of the original module, the word \"Repository\" will be dropped from the type:\n\n```\nFullRepository => Repository :: Repository.full()\n```\n\nThis renaming of the type is automatic for prefixes and suffixes. If no overlap is found, then\nthe full (underscored) schema name will be used for the type:\n\n```\nSimpleUser        => User        :: User.simple()\nPullRequestSimple => PullRequest :: PullRequest.simple()\nMySchema          => Unrelated   :: Unrelated.my_schema()\n```\n\nIf the destination module is later renamed or grouped, the merged schemas will processed in the\nsame way.","ref":"OpenAPI.Processor.Naming.html#merge_schema/2-discussion"},{"type":"function","title":"Configuration - OpenAPI.Processor.Naming.merge_schema/2","doc":"Merges are configured as a list of tuples in the `naming.merge` key of a configuration profile:\n\n    config :oapi_generator, default: [\n      naming: [\n        merge: [\n          {\"PrivateUser\", \"User\"},\n          {~r/Simple$/, \"\"}\n        ]\n      ]\n    ]\n\nIf the first element of the tuple is a string, it will be compared for an exact match to the\nschema's module name. If the first element of the tuple is a regular express, it will be\ncompared to the schema's module name using `Regex.match?/2`. If it matches, the module name will\nbe replaced with the second element of the tuple.\n\nAfter the module name replacement, the type name may be modified. If new the module name is the\nfirst or last part of the original module name, the leftover portion will be used as the type.\nFor example, with the configuration above, the following transformations take place:\n\n    PrivateUser.t() => User.private()\n    UserSimple.t()  => User.simple()\n\nIn the case that the new module name is not a prefix or suffix of the original, the entire\nunderscored original module name is used as the new type.","ref":"OpenAPI.Processor.Naming.html#merge_schema/2-configuration"},{"type":"function","title":"OpenAPI.Processor.Naming.normalize_identifier/2","doc":"Normalize an identifier into CamelCase or snake_case","ref":"OpenAPI.Processor.Naming.html#normalize_identifier/2"},{"type":"function","title":"Example - OpenAPI.Processor.Naming.normalize_identifier/2","doc":"iex> normalize_identifier(\"get-/customer/purchases/{date}_byId\")\n    \"get_customer_purchases_date_by_id\"\n\n    iex> normalize_identifier(\"openAPISpec\", :camel)\n    \"OpenAPISpec\"\n\n    iex> normalize_identifier(\"get-/customer/purchases/{date}_byId\", :lower_camel)\n    \"getCustomerPurchasesDateById\"","ref":"OpenAPI.Processor.Naming.html#normalize_identifier/2-example"},{"type":"function","title":"OpenAPI.Processor.Naming.operation_function/2","doc":"Choose the name of an operation client function based on its operation ID\n\nDefault implementation of `c:OpenAPI.Processor.operation_function_name/2`.\n\nIn this implementation, the operation ID is split up by slash characters with only the last\nportion taken (ex. \"repos/get\" becomes \"get\"), assuming that the module name will use the\nremaining portions. Then the value is normalized to be atom-friendly.\n\nNote that this function creates new atoms, and should not be run in a production environment.","ref":"OpenAPI.Processor.Naming.html#operation_function/2"},{"type":"function","title":"OpenAPI.Processor.Naming.operation_modules/2","doc":"Choose the names of modules containing the given operation\n\nDefault implementation of `c:OpenAPI.Processor.operation_module_names/2`.\n\nThis generator generates a set of modules with functions in them according to some\nnormalization rules:\n\n* Operation tags and IDs are normalized for spaces, slashes, etc.\n* Operation tags are used to generate modules that group operation functions (unless\n  `naming.operation_use_tags` is `false`)\n* Operation IDs with slashes will be split, with the initial segments (everything except the\n  last segment) used as segments of a module\n\nExamples:\n\n  * Operation `foo` with tag `bar` => `Bar.foo`\n  * Operation `foo/bar` with tag `baz` => `Foo.bar` and `Baz.bar`\n  * Operation `foo/bar` without tags => `Foo.bar`\n\nEach operation may exist in multiple modules depending on the quantity of tags and the format\nof the operation ID. If the operation does not have slashes in its ID and does not have any\ntags, then the configured `:default_operation_module` or `[output.base_module].Operations`\nbecomes the module by default.","ref":"OpenAPI.Processor.Naming.html#operation_modules/2"},{"type":"function","title":"Configuration - OpenAPI.Processor.Naming.operation_modules/2","doc":"Use `naming.default_operation_module` to configure the catch-all module name. Note that the\nconfigured name should not include the base module, if it is set in `output.base_module`. The\nfollowing configuration would result in a module named `MyClientLibrary.Operations`:\n\n    config :oapi_generator, default: [\n      naming: [\n        default_operation_module: Operations,\n        operation_use_tags: true\n      ],\n      output: [\n        base_module: MyClientLibrary\n      ]\n    ]\n\nSet `naming.operation_use_tags` to `false` to disable the use of tags when creating modules.","ref":"OpenAPI.Processor.Naming.html#operation_modules/2-configuration"},{"type":"function","title":"OpenAPI.Processor.Naming.raw_schema_module_and_type/3","doc":"Choose a starting schema module and type name based on title and context\n\nReturns a tuple containing the `{module, type}`, such as `{\"MySchema\", \"t\"}`.\n\nThis function does not consider schema renaming or merging. It uses the title, context, and\nlocation of the schema within the specification to determine an initial set of names. Schemas\nlocated in `components/schemas` are named based on their key in the `schemas` map, so a schema\nlocated at `components/schemas/my_schema` will become `MySchema.t()`. If a schema has a\ncontext attached (such as a request body or response body for an operation) then it will be\nnamed based on the operation. Finally, if a schema has a defined title, this will be used as\nthe name. If none of this information is available, `{nil, \"map\"}` is returned.\n\nCallers of this function will almost certainly want to perform further processing.","ref":"OpenAPI.Processor.Naming.html#raw_schema_module_and_type/3"},{"type":"function","title":"OpenAPI.Processor.Naming.readable_content_type/1","doc":"Turn a content type (ex. `\"application/json\"`) into a readable type (ex. `\"json\"`)\n\nThis is used by the default implementation of the schema module/type name function while\nconstructing the type of a request or response body that is otherwise unnamed. If an unknown\ncontent type is passed, this function returns an empty string to avoid including the content\ntype in the name (although this could cause collisions).","ref":"OpenAPI.Processor.Naming.html#readable_content_type/1"},{"type":"function","title":"OpenAPI.Processor.Naming.rename_schema/2","doc":"Rename schema modules based on configured patterns\n\nThis function accepts a tuple with the module and type of a schema as strings, along with the\nprocessor state, and returns a modified tuple according to the configured replacements.","ref":"OpenAPI.Processor.Naming.html#rename_schema/2"},{"type":"function","title":"Configuration - OpenAPI.Processor.Naming.rename_schema/2","doc":"Module replacements can be configured as a list of tuples in the `naming.rename` key of a\nconfiguration profile:\n\n    config :oapi_generator, default: [\n      naming: [\n        rename: [\n          {\"Api\", \"API\"},\n          {~r/^Bio/, \"Author.Bio\"},\n          # ...\n        ]\n      ]\n    ]\n\nThe contents of each tuple will be fed into `String.replace/3`, for example:\n\n    > String.replace(\"MyApiResponse\", \"Api\", \"API\")","ref":"OpenAPI.Processor.Naming.html#rename_schema/2-configuration"},{"type":"function","title":"Examples - OpenAPI.Processor.Naming.rename_schema/2","doc":"In the configuration above, there are two replacements configured: the string pattern `\"Api\"`\nwill be replaced with `\"API\"`, and the regular expression pattern `^Bio` will be replaced with\n`\"Author.Bio\"`. These rules would create the following transformations (types omitted because\nthey do not change):\n\n    MyApiResponse => MyAPIResponse\n    Apiary        => APIary\n    BioUpdate     => Author.BioUpdate\n    EditorBio     => EditorBio\n\nNote that replacements can have unintended side-effects. For example, while we correctly\ncapitalized `MyApiResponse` using the `\"Api\"` pattern, we also replaced `APIary`. Regular\nexpressions lend more powerful and precise replacement patterns. This includes the ability to\nuse capture expressions (ex. `~r/(Api)([A-Z]|$)/`) and replacements that reference those\ncaptures (ex. `\"API\\\\2\"`). See `String.replace/3` for more information.","ref":"OpenAPI.Processor.Naming.html#rename_schema/2-examples"},{"type":"function","title":"OpenAPI.Processor.Naming.schema_module_and_type/2","doc":"Choose the name of the schema's module and type\n\nDefault implementation of `c:OpenAPI.Processor.schema_module_and_type/2`.\n\nMost of the configuration of this project relates to the manipulation of schema names. It is\nimportant to understand the order of operations. As an example, imagine an OpenAPI description\nhas the following schemas:\n\n  * `#/components/schemas/simple-user`\n  * `#/components/schemas/user`\n  * `#/components/schemas/user-preferences`\n\nAnd the following configuration:\n\n    config :oapi_generator, default: [\n      naming: [\n        group: [User],\n        merge: [{\"SimpleUser\", \"User\"}]\n        rename: [{~r/Preferences/, \"Settings\"}]\n      ],\n      output: [\n        base_module: Example\n      ]\n    ]\n\nIn this case, naming would proceed as follows:\n\n1. Schemas in the OpenAPI descriptions are turned into Elixir modules based on their location,\n  context, or title by `raw_schema_module_and_type/1`:\n  ```\n#/components/schemas/simple-user       =>  SimpleUser.t()\n#/components/schemas/user              =>  User.t()\n#/components/schemas/user-preferences  =>  UserPreferences.t()\n  ```\n\n2. Merge settings are applied based on the original names of the schemas by `merge_schema/2`:\n  ```\nSimpleUser.t()  =>  User.simple()\n  ```\n\n3. Rename settings are applied based on the merged module names by `rename_schema/2`:\n  ```\nUserPreferences.t()  =>  UserSettings.t()\n  ```\n\n4. Group settings are applied based on the renamed module names by `group_schema/2`:\n  ```\nUserSettings.t()  =>  User.Settings.t()\n  ```\n\n5. The base module is applied to get the final names:\n  ```\nUser.simple()      =>  Example.User.simple()\nUser.t()           =>  Example.User.t()\nUser.Settings.t()  =>  Example.User.Settings.t()\n  ```","ref":"OpenAPI.Processor.Naming.html#schema_module_and_type/2"},{"type":"function","title":"Collapsing - OpenAPI.Processor.Naming.schema_module_and_type/2","doc":"Note that `User.simple()` and `User.t()` will end up in the same file as a result of the merge,\nsharing the same struct for their responses (with distinct typespecs).","ref":"OpenAPI.Processor.Naming.html#schema_module_and_type/2-collapsing"},{"type":"type","title":"OpenAPI.Processor.Naming.module_and_type/0","doc":"","ref":"OpenAPI.Processor.Naming.html#t:module_and_type/0"},{"type":"type","title":"OpenAPI.Processor.Naming.raw_module_and_type/0","doc":"","ref":"OpenAPI.Processor.Naming.html#t:raw_module_and_type/0"},{"type":"module","title":"OpenAPI.Processor.Operation","doc":"Default plugin for formatting operations\n\nThis module also provides the Operation struct that is used by the renderer.","ref":"OpenAPI.Processor.Operation.html"},{"type":"function","title":"OpenAPI.Processor.Operation.docstring/3","doc":"Create the contents of an `@doc` string for the given operation\n\nDefault implementation of `c:OpenAPI.Processor.operation_docstring/3`.\n\nThe docstring constructed by this function will contain a summary line provided by the operation\nsummary (if available) or the request method and path otherwise. It will incorporate the\noperation description (if available) and link to any included external documentation. Finally,\nall query parameters (which are part of the `opts` argument) are documented.\n\n    @doc \"\"\"\n    Summary of the operation or method and path\n\n    Description of the operation, which generally provides more information.","ref":"OpenAPI.Processor.Operation.html#docstring/3"},{"type":"function","title":"Options - OpenAPI.Processor.Operation.docstring/3","doc":"* `param`: query parameter description","ref":"OpenAPI.Processor.Operation.html#docstring/3-options"},{"type":"function","title":"Resources - OpenAPI.Processor.Operation.docstring/3","doc":"* [External Doc Description](link to external documentation)\n\n    \"\"\"","ref":"OpenAPI.Processor.Operation.html#docstring/3-resources"},{"type":"function","title":"OpenAPI.Processor.Operation.request_body/2","doc":"Collect request content types and their associated schemas\n\nDefault implementation of `c:OpenAPI.Processor.operation_request_body/2`.","ref":"OpenAPI.Processor.Operation.html#request_body/2"},{"type":"function","title":"OpenAPI.Processor.Operation.request_method/2","doc":"Cast the HTTP method to an atom\n\nDefault implementation of `c:OpenAPI.Processor.operation_request_method/2`.","ref":"OpenAPI.Processor.Operation.html#request_method/2"},{"type":"function","title":"OpenAPI.Processor.Operation.response_body/2","doc":"Collect response status codes and their associated schemas\n\nDefault implementation of `c:OpenAPI.Processor.operation_response_body/2`.\n\nIn this implementation, all schemas are returned regardless of content type. It is possible for\nthe same status code to have multiple schemas, in which case the renderer should compose a\nunion type for the response.","ref":"OpenAPI.Processor.Operation.html#response_body/2"},{"type":"type","title":"OpenAPI.Processor.Operation.method/0","doc":"HTTP method","ref":"OpenAPI.Processor.Operation.html#t:method/0"},{"type":"type","title":"OpenAPI.Processor.Operation.request_body/0","doc":"Request content types and their associated schemas","ref":"OpenAPI.Processor.Operation.html#t:request_body/0"},{"type":"type","title":"OpenAPI.Processor.Operation.request_body_unprocessed/0","doc":"Request content types and their associated schema specs","ref":"OpenAPI.Processor.Operation.html#t:request_body_unprocessed/0"},{"type":"type","title":"OpenAPI.Processor.Operation.response_body/0","doc":"Response status codes and their associated schemas","ref":"OpenAPI.Processor.Operation.html#t:response_body/0"},{"type":"type","title":"OpenAPI.Processor.Operation.response_body_unprocessed/0","doc":"Response status codes and their associated schema specs","ref":"OpenAPI.Processor.Operation.html#t:response_body_unprocessed/0"},{"type":"type","title":"OpenAPI.Processor.Operation.t/0","doc":"Processed operation data used by the renderer","ref":"OpenAPI.Processor.Operation.html#t:t/0"},{"type":"module","title":"OpenAPI.Processor.Operation.Param","doc":"Provides the Param struct that is used by the renderer","ref":"OpenAPI.Processor.Operation.Param.html"},{"type":"type","title":"OpenAPI.Processor.Operation.Param.location/0","doc":"Location of the param","ref":"OpenAPI.Processor.Operation.Param.html#t:location/0"},{"type":"type","title":"OpenAPI.Processor.Operation.Param.t/0","doc":"Processed param data used by the renderer","ref":"OpenAPI.Processor.Operation.Param.html#t:t/0"},{"type":"module","title":"OpenAPI.Processor.Schema","doc":"Processed schema used by the renderer\n\nThis struct is created by the Processor to hold only the data necessary for rendering schemas\nand their types. It has the following fields:\n\n  * `context`: List of contexts where the schema is found in the API description.\n  * `fields`: List of `t:OpenAPI.Processor.Schema.Field.t/0` structs contained in the schema.\n  * `module_name`: Name of the module where the schema will be defined.\n  * `output_format`: Intended format of the output (ex. struct or typespec).\n  * `ref`: Reference of the schema and its original spec in the processor state.\n  * `type_name`: Name of the schema's type within its module.\n\nAll of this data is managed by the code generator, and it is unlikely that a callback would\nneed to modify this struct directly.","ref":"OpenAPI.Processor.Schema.html"},{"type":"type","title":"OpenAPI.Processor.Schema.format/0","doc":"Format of rendering the schema (full struct or inline typespec)","ref":"OpenAPI.Processor.Schema.html#t:format/0"},{"type":"type","title":"OpenAPI.Processor.Schema.t/0","doc":"Processed schema used by the renderer","ref":"OpenAPI.Processor.Schema.html#t:t/0"},{"type":"module","title":"OpenAPI.Processor.Schema.Field","doc":"Provides the Field struct that is used by the renderer\n\nThis struct is created by the Processor to hold only the data necessary for rendering fields\nand their types. It has the following fields:\n\n  * `name`: Name of the field in its parent schema\n  * `nullable`: Whether the field is defined as nullable\n  * `private`: Whether the field was added via the `output.extra_fields` configuration\n  * `required`: Whether the field is marked as required by its parent schema\n  * `type`: Internal representation of the field's type","ref":"OpenAPI.Processor.Schema.Field.html"},{"type":"type","title":"OpenAPI.Processor.Schema.Field.t/0","doc":"Processed field data used by the renderer","ref":"OpenAPI.Processor.Schema.Field.html#t:t/0"},{"type":"module","title":"OpenAPI.Processor.State","doc":"State of the process phase of code generation\n\nThis struct is created at the beginning of the process phase using data from the overall\ngenerator `OpenAPI.State`. It has the following fields:\n\n  * `implementation`: Module configured as the implementation of the processor (defaults to\n    `OpenAPI.Processor`). Callbacks must use this field when calling other process callbacks.\n  * `operations`: Cumulative list of `t:OpenAPI.Processor.Operation.t/0` structs identified\n    and processed so far.\n  * `profile`: Name of the active configuration profile. Callbacks must use this field when\n    looking up configuration from the application environment.\n  * `schema_specs_by_path`: Map of raw schema specifications keyed by the original paths where\n    they could be found in the spec files, as provided by the read phase. This map may include\n    schemas that are not used in operations.\n  * `schema_specs_by_ref`: Cumulative map of raw schema specifications keyed by the internal\n    `ref`s used to refer to them in types.\n  * `schema_refs_by_path`: Cumulative map of internal `ref`s used to refer to schemas keyed by\n    the original paths where they could be found in the spec files.\n  * `schemas_by_ref`: Cumulative map of processed `t:OpenAPI.Processor.Schema.t/0` structs\n    keyed by the internal `ref`s used to refer to them in types, as they are processed.\n  * `spec`: The original parsed OpenAPI description.\n\nAll of this data is managed by the code generator, and it is unlikely that a callback would\nneed to transform this struct directly.","ref":"OpenAPI.Processor.State.html"},{"type":"function","title":"OpenAPI.Processor.State.put_new_schema/3","doc":"Add a processed schema to the processor state by its reference only if is is not already present","ref":"OpenAPI.Processor.State.html#put_new_schema/3"},{"type":"function","title":"OpenAPI.Processor.State.put_schema/3","doc":"Add a processed schema to the processor state by its reference","ref":"OpenAPI.Processor.State.html#put_schema/3"},{"type":"function","title":"OpenAPI.Processor.State.put_schema_spec/2","doc":"Add a schema spec to the processor state and generate a reference for it","ref":"OpenAPI.Processor.State.html#put_schema_spec/2"},{"type":"type","title":"OpenAPI.Processor.State.t/0","doc":"","ref":"OpenAPI.Processor.State.html#t:t/0"},{"type":"module","title":"OpenAPI.Processor.Type","doc":"","ref":"OpenAPI.Processor.Type.html"},{"type":"function","title":"OpenAPI.Processor.Type.from_schema/2","doc":"Create an internal type representation of the given schema\n\nThis function supports referencing object schemas in the processor state. As a result, if it\nencounters an object schema as a sub-field of the given schema, it will potentially modify the\nprocessor state in order to stash a new reference.","ref":"OpenAPI.Processor.Type.html#from_schema/2"},{"type":"function","title":"OpenAPI.Processor.Type.reduce/3","doc":"","ref":"OpenAPI.Processor.Type.html#reduce/3"},{"type":"type","title":"OpenAPI.Processor.Type.literal/0","doc":"Literal values, as found in `const` and `enum` definitions","ref":"OpenAPI.Processor.Type.html#t:literal/0"},{"type":"type","title":"OpenAPI.Processor.Type.primitive/0","doc":"Basic type\n\nThese types represent a subset of the six primitive values defined by JSON schema (omitting\nobject and array, which are represented in `t:t/0`.) Note that both `integer` and `number` are\nacceptable types, with the latter covering both integer and decimal numbers.","ref":"OpenAPI.Processor.Type.html#t:primitive/0"},{"type":"type","title":"OpenAPI.Processor.Type.string_format/0","doc":"Special cases of the string type\n\n`binary` represents binary data that may be encoded, while `generic` represents a generic string\nwithout a formatting constraint.","ref":"OpenAPI.Processor.Type.html#t:string_format/0"},{"type":"type","title":"OpenAPI.Processor.Type.t/0","doc":"Internal representation of types\n\nIn addition to the `t:primitive/0` types, values can be constants or enums defined by\n`t:literal/0` values. They can also be arrays or objects, as well as the union of multiple\ntype definitions. If a value is nullable, it will be expressed as the union with `:null`.\n\nElixir references (`t:reference/0`) represent schemas stored in the processor `State`.","ref":"OpenAPI.Processor.Type.html#t:t/0"},{"type":"type","title":"OpenAPI.Processor.Type.unnatural/0","doc":"Unnatural types introduced by the generator\n\nThese types may be introduced in extra fields or when referenced schemas are ignored.","ref":"OpenAPI.Processor.Type.html#t:unnatural/0"},{"type":"behaviour","title":"OpenAPI.Renderer","doc":"Phase three of code generation\n\nThe **render** phase begins with operation and schema structs created during the **process**\nphase. It uses this data to construct and save files containing Elixir source code. Most of\nthe work done by this phase involves the manipulation of Elixir ASTs.","ref":"OpenAPI.Renderer.html"},{"type":"callback","title":"OpenAPI.Renderer.format/2","doc":"Convert the Abstract Syntax Tree (AST) form of the file into formatted code\n\nThis callback can expect a `t:OpenAPI.Renderer.File.t/0` struct with the completed contents of\nthe file included in the `ast` field. Nodes of the AST may include optional formatting metadata\n(ex. `delimiter`, `indentation`, or `end_of_expression`). It is recommended that the formatter\nadhere to the standard configuration of the default Mix formatter (for example, formatting to a\nline width of 98) in order to avoid a large amount of changes should someone run `mix format` on\nthe generated code.\n\nThe return value of the callback can be `iodata` (strings do not need to be concatenated), and\nit will be stored in the `contents` field of the file.\n\nSee `OpenAPI.Renderer.Util.format/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:format/2"},{"type":"callback","title":"OpenAPI.Renderer.location/2","doc":"Choose the filesystem location for a rendered file to be written\n\nSee `OpenAPI.Renderer.Module.filename/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:location/2"},{"type":"callback","title":"OpenAPI.Renderer.render/2","doc":"Create the contents of a file in quoted Abstract Syntax Tree (AST) form\n\nThis callback is the primary function called to render a file. The default implementation calls\nseveral other callbacks, each of which my be overridden separately:\n\n  * `c:OpenAPI.Renderer.render_moduledoc/2`\n  * `c:OpenAPI.Renderer.render_using/2`\n  * `c:OpenAPI.Renderer.render_default_client/2`\n  * `c:OpenAPI.Renderer.render_schema/2`\n  * `c:OpenAPI.Renderer.render_operations/2`\n\nSee `OpenAPI.Renderer.Module.render/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:render/2"},{"type":"callback","title":"OpenAPI.Renderer.render_default_client/2","doc":"Render the `@default_client` module attribute in an operation module\n\nWhen using the default operation function renderer, every operation function includes a line:\n\n    client = opts[:client] || @default_client\n\nThis allows callers to override the client implementation without having to pass the default\nmodule in as an argument. This callback renders the definition of the `@default_client` module\nattribute, effectively choosing which module will be called for every operation.\n\nSee `OpenAPI.Renderer.Module.render_default_client/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:render_default_client/2"},{"type":"callback","title":"OpenAPI.Renderer.render_moduledoc/2","doc":"Render the `@moduledoc` portion of the file\n\nUsers of a client library may lean on this documentation to find the operation or schema they\nneed. While the default implementation presents a fairly basic line of documentation depending\non whether the file contains operations or a schema, custom implementations of this callback\ncould provide rich and helpful instructions to consumers.\n\nSee `OpenAPI.Renderer.Module.render_moduledoc/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:render_moduledoc/2"},{"type":"callback","title":"OpenAPI.Renderer.render_operation/2","doc":"Render the associated types, docstring, typespec, and function for a single operation\n\nThe default implementation of this function calls several other callbacks (all named\n`render_operation_*`) which can be overridden individually.\n\nSee `OpenAPI.Renderer.Operation.render/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:render_operation/2"},{"type":"callback","title":"OpenAPI.Renderer.render_operation_doc/2","doc":"Render the `@doc` portion of an operation function\n\nSee `OpenAPI.Renderer.Operation.render_doc/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:render_operation_doc/2"},{"type":"callback","title":"OpenAPI.Renderer.render_operation_function/2","doc":"Render the main function portion of an operation\n\nSee `OpenAPI.Renderer.Operation.render_function/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:render_operation_function/2"},{"type":"callback","title":"OpenAPI.Renderer.render_operation_spec/2","doc":"Render the `@spec` portion of an operation function\n\nSee `OpenAPI.Renderer.Operation.render_spec/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:render_operation_spec/2"},{"type":"callback","title":"OpenAPI.Renderer.render_operations/2","doc":"Render the associated types, docstring, typespec, and function for all operations\n\nThis is the primary function called to render all operations in a file. The default\nimplementation calls several other callbacks (all named `render_operation*`) which can be\noverridden individually.\n\nSee `OpenAPI.Renderer.Operation.render_all/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:render_operations/2"},{"type":"callback","title":"OpenAPI.Renderer.render_schema/2","doc":"Render the types, struct, and field function for schemas not related to an operation\n\nThis is the primary function called to render schemas. The default implementation calls several\nother callbacks (all named `render_schema_*`) which can be overridden individually.\n\nSee `OpenAPI.Renderer.Schema.render/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:render_schema/2"},{"type":"callback","title":"OpenAPI.Renderer.render_schema_field_function/2","doc":"Render a function `__fields__/1` that return a keyword list of schema fields and their types\n\nSee `OpenAPI.Renderer.Schema.render_field_function/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:render_schema_field_function/2"},{"type":"callback","title":"OpenAPI.Renderer.render_schema_struct/2","doc":"Render the `defstruct` call for the schema types contained in the file\n\nSee `OpenAPI.Renderer.Schema.render_struct/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:render_schema_struct/2"},{"type":"callback","title":"OpenAPI.Renderer.render_schema_types/2","doc":"Render the typespecs for schema types contained in the file\n\nSee `OpenAPI.Renderer.Schema.render_types/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:render_schema_types/2"},{"type":"callback","title":"OpenAPI.Renderer.render_using/2","doc":"Render one or more `use` statements to include in the file\n\nAnother route for customization of the outputted code is via meta-programming. This callback\nenables library authors to `use` any module they like at the top of files that contain schemas,\noperations, or both. The referenced modules can then perform additional compile-time changes.\n\nSee `OpenAPI.Renderer.Module.render_using/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:render_using/2"},{"type":"function","title":"OpenAPI.Renderer.run/1","doc":"","ref":"OpenAPI.Renderer.html#run/1"},{"type":"callback","title":"OpenAPI.Renderer.write/2","doc":"Write a rendered file to the filesystem\n\nThis callback can expect to receive a `t:OpenAPI.Renderer.File.t/0` struct with formatted file\ncontents expressed as `iodata` in the `contents` field. It should write the file to the\nfilesystem at the appropriate location included in the `location` field. While the return value\nis irrelevant, a simple `:ok` will suffice.\n\nSee `OpenAPI.Renderer.Util.write/2` for the default implementation.","ref":"OpenAPI.Renderer.html#c:write/2"},{"type":"module","title":"OpenAPI.Renderer.File","doc":"Collection of operations and schemas that belong to the same file\n\nAs rendering occurs for a particular file, this struct will collect all of the necessary data\nto continue the process. It has the following fields:\n\n  * `ast`: Once rendered, the Abstract Syntax Tree (AST), or quoted form, of the file contents.\n    This AST may include additional formatting-related metadata.\n  * `contents`: Once formatted, the `iodata` form of the file contents. This is what will be\n    used by the default implementation of the write callback.\n  * `location`: Once determined, the filesystem location (path) where the file is intended to\n    be written.\n  * `module`: Name of the Elixir module contained in the file.\n  * `operations`: List of `t:OpenAPI.Processor.Operation.t/0` contained in the file.\n  * `schemas`: List of `t:OpenAPI.Processor.Schema.t/0` contained in the file.\n\nAll of this state is managed by the code generator, so it is unlikely that callbacks would need\nto write to this struct directly.","ref":"OpenAPI.Renderer.File.html"},{"type":"type","title":"OpenAPI.Renderer.File.t/0","doc":"File contents\n\nSee module documentation for additional information.","ref":"OpenAPI.Renderer.File.html#t:t/0"},{"type":"module","title":"OpenAPI.Renderer.Module","doc":"Default implementation for callbacks related to rendering operation and schema modules\n\nThis module contains the default implementations for:\n\n  * `c:OpenAPI.Renderer.location/2`\n  * `c:OpenAPI.Renderer.render/2`\n  * `c:OpenAPI.Renderer.render_default_client/2`\n  * `c:OpenAPI.Renderer.render_moduledoc/2`\n  * `c:OpenAPI.Renderer.render_using/2`\n\nThese focus on portions of modules that may appear in both schema and operation modules.","ref":"OpenAPI.Renderer.Module.html"},{"type":"module","title":"Configuration - OpenAPI.Renderer.Module","doc":"All configuration offered by the functions in this module lives under the `output` key of the\nactive configuration profile. For example (default values shown):\n\n    # config/config.exs\n\n    config :oapi_generator, default: [\n      output: [\n        base_module: nil,\n        default_client: Client,\n        location: \"\",\n        operation_subdirectory: \"\",\n        operation_use: nil,\n        schema_subdirectory: \"\",\n        schema_use: nil\n      ]\n    ]","ref":"OpenAPI.Renderer.Module.html#module-configuration"},{"type":"function","title":"OpenAPI.Renderer.Module.filename/2","doc":"Choose the filesystem location for the given file based on its module name and contents\n\nDefault implementation of `c:OpenAPI.Renderer.location/2`.\n\nIf the file does not contain any operations, the file name is chosen as the concatenation of:\n\n* The base `location` set in the `output` configuration,\n* The `schema_subdirectory` set in the `output` configuration, and\n* The underscored module name (ex. `Some.Schema` becomes `some/schema.ex`)","ref":"OpenAPI.Renderer.Module.html#filename/2"},{"type":"function","title":"Configuration - OpenAPI.Renderer.Module.filename/2","doc":"Use `output.location` to set the directory of all outputted files (ex. `lib`). Then, optionally\nsplit operations and schemas into separate directories using `output.operation_subdirectory`\nand `output.schema_subdirectory`. This can be useful to show which modules are generated vs.\nthose that are written by hand.\n\n    config :oapi_generator, default: [\n      output: [\n        location: \"lib\",\n        operation_subdirectory: \"operations\",\n        schema_subdirectory: \"schemas\"\n      ]\n    ]\n\nWith this configuration, an schema module named `My.ExampleSchema` would be output to\n`lib/schemas/my/example_schema.ex`.","ref":"OpenAPI.Renderer.Module.html#filename/2-configuration"},{"type":"function","title":"OpenAPI.Renderer.Module.render/2","doc":"Create the contents of a file in quoted Abstract Syntax Tree (AST) form\n\nDefault implementation of `c:OpenAPI.Renderer.render/2`.\n\nThis callback is the primary function called to render a file. It makes use of several other\ncallbacks of the renderer (in this order), each of which my be overridden separately:\n\n  * `c:OpenAPI.Renderer.render_moduledoc/2`\n  * `c:OpenAPI.Renderer.render_using/2`\n  * `c:OpenAPI.Renderer.render_default_client/2`\n  * `c:OpenAPI.Renderer.render_schema/2`\n  * `c:OpenAPI.Renderer.render_operations/2`\n\nBesides concatenating the results of these functions, this function also writes the `defmodule`\ncall itself.","ref":"OpenAPI.Renderer.Module.html#render/2"},{"type":"function","title":"Configuration - OpenAPI.Renderer.Module.render/2","doc":"Use `output.base_module` to determine a prefix to the name of all modules created by the\ngenerator. For example, given the following configuration:\n\n    config :oapi_generator, default: [\n      output: [\n        base_module: MyClientLibrary\n      ]\n    ]\n\nA file with module name `MySchema` would be output as `MyClientLibrary.MySchema`. Usually, this\nconfiguration contains the name of your library's root module.","ref":"OpenAPI.Renderer.Module.html#render/2-configuration"},{"type":"function","title":"OpenAPI.Renderer.Module.render_default_client/2","doc":"Construct the `@default_client` module attribute for modules with operations\n\nDefault implementation of `c:OpenAPI.Renderer.render_default_client/2`.\n\nThis allows callers to override the client implementation without having to pass the default\nmodule in as an argument. This callback renders the definition of the `@default_client` module\nattribute, effectively choosing which module will be called for every operation.","ref":"OpenAPI.Renderer.Module.html#render_default_client/2"},{"type":"function","title":"Configuration - OpenAPI.Renderer.Module.render_default_client/2","doc":"Use `output.default_client` to choose which module is set. A value of `false` or `nil` will\ncause the module attribute not to be set at all, which may cause compilation errors if using\nthe default implementation of the operation function renderer.\n\n    config :oapi_generator, default: [\n      output: [\n        default_client: MyLib.MyClient\n      ]\n    ]\n\nThis will result in a statement `@default_client MyLib.MyClient` to be added to any module that\ncontains operations. Note that the `output.base_module` configuration is not used in this case.\n\nIf the configuration is unset, then a default module of `[base module].Client` will be used,\nbased on the `output.base_module` configuration.","ref":"OpenAPI.Renderer.Module.html#render_default_client/2-configuration"},{"type":"function","title":"OpenAPI.Renderer.Module.render_moduledoc/2","doc":"Construct the `@moduledoc` portion of the file based on the file contents\n\nDefault implementation of `c:OpenAPI.Renderer.render_moduledoc/2`.\n\nThis function provides a basic moduledoc for each file. If the file contains only schemas, then\nthe moduledoc focuses on the structs and types it provides:\n\n> Provides struct and types for a MySchema\n\nIf the file contains operations, it focuses on those:\n\n> Provides API endpoints related to MyOperations","ref":"OpenAPI.Renderer.Module.html#render_moduledoc/2"},{"type":"function","title":"OpenAPI.Renderer.Module.render_using/2","doc":"Construct any `use` statements that should be included in the file\n\nDefault implementation of `c:OpenAPI.Renderer.render_using/2`.\n\nAnother route for customization of the outputted code is via meta-programming. This callback\nenables library authors to `use` any module they like at the top of files that contain schemas,\noperations, or both. The referenced modules can then perform additional compile-time changes.","ref":"OpenAPI.Renderer.Module.html#render_using/2"},{"type":"function","title":"Configuration - OpenAPI.Renderer.Module.render_using/2","doc":"Use `output.operation_use` to include a `use` statement at the top of each file that contains\nany operations, and `output.schema_use` to similarly include a `use` statement at the top of\nfiles containing a schema. Keep in mind that some files may contain both operations and a\nschema.\n\n    config :oapi_generator, default: [\n      output: [\n        schema_use: MyLib.Schema\n      ]\n    ]\n\nThis will result in a statement `use MyLib.Schema` (note that the `output.base_module`\nconfiguration is not used in this context).","ref":"OpenAPI.Renderer.Module.html#render_using/2-configuration"},{"type":"module","title":"OpenAPI.Renderer.Operation","doc":"Default implementation for callbacks related to rendering operations\n\nThis module contains the default implementations for:\n\n  * `c:OpenAPI.Renderer.render_operations/2`\n  * `c:OpenAPI.Renderer.render_operation/2`\n  * `c:OpenAPI.Renderer.render_operation_doc/2`\n  * `c:OpenAPI.Renderer.render_operation_function/2`\n  * `c:OpenAPI.Renderer.render_operation_spec/2`\n\nThese focus on the operation functions and surrounding code.","ref":"OpenAPI.Renderer.Operation.html"},{"type":"module","title":"Configuration - OpenAPI.Renderer.Operation","doc":"All configuration offered by the functions in this module lives under the `output` key of the\nactive configuration profile. For example (default values shown):\n\n    # config/config.exs\n\n    config :oapi_generator, default: [\n      output: [\n        base_module: nil,\n        operation_call: [\n          request: :list\n        ],\n        types: [\n          error: nil\n        ]\n      ]\n    ]","ref":"OpenAPI.Renderer.Operation.html#module-configuration"},{"type":"function","title":"OpenAPI.Renderer.Operation.render/2","doc":"Render a single operation\n\nDefault implementation of `c:OpenAPI.Renderer.render_operation/2`.\n\nThis implementation calls the following callbacks and concatenates their results:\n\n  * `c:OpenAPI.Renderer.render_operation_doc/2`\n  * `c:OpenAPI.Renderer.render_operation_spec/2`\n  * `c:OpenAPI.Renderer.render_operation_function/2`","ref":"OpenAPI.Renderer.Operation.html#render/2"},{"type":"function","title":"OpenAPI.Renderer.Operation.render_all/2","doc":"Render all of the operations contained in a single module\n\nDefault implementation of `c:OpenAPI.Renderer.render_operations/2`.\n\nThis implementation iterates through the operations contained in a file, sorted by their\nfunction name, and calls both `c:OpenAPI.Renderer.render_operation/2` and\n`c:OpenAPI.Renderer.render_schema_types/2` callbacks for each. The latter is only given schemas\nthat have an output format of `:typed_map` and relate exclusively to the operation. Afterwards,\nit calls `c:OpenAPI.Renderer.render_schema_field_function/2` for all schemas that had types\noutput earlier. It returns a list of AST nodes.","ref":"OpenAPI.Renderer.Operation.html#render_all/2"},{"type":"function","title":"OpenAPI.Renderer.Operation.render_call/2","doc":"Render a call to `client.request/1` in the body of an operation function\n\nThis function is called by the default implementation of\n`c:OpenAPI.Renderer.render_operation_function/2` (see `render_function/2`). It returns code\nsimilar to this:\n\n    client.request(%{...})\n\n**Warning**: This function is public for the benefit of plugin implementers who wish to\nreplicate portions of the default implementation. It is subject to change.","ref":"OpenAPI.Renderer.Operation.html#render_call/2"},{"type":"function","title":"OpenAPI.Renderer.Operation.render_call_request_info/3","doc":"Renders a keyword list element containing information about the request body\n\nThe second argument accepts a format for the output code, which can be `:map` or the default of\n`:list`.\n\nThis function is called by the default implementation of\n`c:OpenAPI.Renderer.render_operation_function/2` (see `render_function/2`). It returns code\nsimilar to this:\n\n    # Default format:\n    request: [{\"application/json\", {MySchema, :t}}]\n\n    # With format `:map`:\n    request: %{\"application/json\" => {MySchema, :t}}\n\n**Warning**: This function is public for the benefit of plugin implementers who wish to\nreplicate portions of the default implementation. It is subject to change.","ref":"OpenAPI.Renderer.Operation.html#render_call_request_info/3"},{"type":"function","title":"OpenAPI.Renderer.Operation.render_doc/2","doc":"Render the docstring for an operation function\n\nDefault implementation of `c:OpenAPI.Renderer.render_operation_doc/2`.\n\nThis implementation uses the docstring created by the processor without modification.","ref":"OpenAPI.Renderer.Operation.html#render_doc/2"},{"type":"function","title":"OpenAPI.Renderer.Operation.render_function/2","doc":"Render the function definition for an operation function\n\nDefault implementation of `c:OpenAPI.Renderer.render_operation_function/2`.\n\nThis implementation constructs a function that calls a dynamically chosen client module's\n`request` function with details about the operation.","ref":"OpenAPI.Renderer.Operation.html#render_function/2"},{"type":"function","title":"Configuration - OpenAPI.Renderer.Operation.render_function/2","doc":"Use `output.operation_call` to modify the format of output code within the function call. For\nexample, the following will output a map for the request body information:\n\n    config :oapi_generator, default: [\n      output: [\n        operation_call: [\n          request: :map\n        ]\n      ]\n    ]","ref":"OpenAPI.Renderer.Operation.html#render_function/2-configuration"},{"type":"function","title":"Example - OpenAPI.Renderer.Operation.render_function/2","doc":"def my_operation(path_param, body, opts \\ []) do\n        client = opts[:client] || @default_client\n        query = Keyword.take(opts, [:query_param])\n\n        client.request(%{\n          args: [path_param: path_param, body: body],\n          call: {Example.Operations, :my_operation},\n          url: \"/path/to/#{path_param}\",\n          body: body,\n          method: :post,\n          query: query,\n          request: [{\"application/json\", :map}],\n          response: [{200, :map}, {404, {Example.NotFoundError, :t}}],\n          opts: opts\n        })\n      end","ref":"OpenAPI.Renderer.Operation.html#render_function/2-example"},{"type":"function","title":"OpenAPI.Renderer.Operation.render_query/1","doc":"Render code to handle query params in the body of an operation function\n\nThis function is called by the default implementation of\n`c:OpenAPI.Renderer.render_operation_function/2` (see `render_function/2`). It returns code\nsimilar to this:\n\n    query = Keyword.take(opts, [:param1, :param2])\n\n**Warning**: This function is public for the benefit of plugin implementers who wish to\nreplicate portions of the default implementation. It is subject to change.","ref":"OpenAPI.Renderer.Operation.html#render_query/1"},{"type":"function","title":"OpenAPI.Renderer.Operation.render_spec/2","doc":"Render the spec of an operation function\n\nDefault implementation of `c:OpenAPI.Renderer.render_operation_spec/2`.","ref":"OpenAPI.Renderer.Operation.html#render_spec/2"},{"type":"module","title":"OpenAPI.Renderer.Schema","doc":"Default implementation for callbacks related to rendering schemas\n\nThis module contains the default implementations for:\n\n  * `c:OpenAPI.Renderer.render_schema/2`\n  * `c:OpenAPI.Renderer.render_schema_field_function/2`\n  * `c:OpenAPI.Renderer.render_schema_struct/2`\n  * `c:OpenAPI.Renderer.render_schema_types/2`","ref":"OpenAPI.Renderer.Schema.html"},{"type":"module","title":"Extra Fields - OpenAPI.Renderer.Schema","doc":"It is sometimes useful for client libraries to store additional information on the structs\ndefined by the API description. This kind of private information is supported using the\n`output.extra_fields` configuration. Each entry in the keyword list must use the name of the\nfield as the atom key and the type as the value. Types can be any valid\n`t:OpenAPI.Processor.Type.t/0`.\n\nExample:\n\n    config :oapi_generator, default: [\n      output: [\n        __info__: :map\n      ]\n    ]\n\nThis will result in a field `info: map` added to every schema typespec and `:info` added to\nevery struct definition.","ref":"OpenAPI.Renderer.Schema.html#module-extra-fields"},{"type":"module","title":"Configuration - OpenAPI.Renderer.Schema","doc":"All configuration offered by the functions in this module lives under the `output` key of the\nactive configuration profile. For example (default values shown):\n\n    # config/config.exs\n\n    config :oapi_generator, default: [\n      output: [\n        extra_fields: []\n      ]\n    ]","ref":"OpenAPI.Renderer.Schema.html#module-configuration"},{"type":"function","title":"OpenAPI.Renderer.Schema.render/2","doc":"Render a schema, including typespecs, struct definition, and field function\n\nDefault implementation of `c:OpenAPI.Renderer.render_schema/2`.\n\nThis implementation operates on schemas that appear in multiple contexts. If a schema appears\nin exactly one context as the request or response body of a function, it will instead be\nrendered by an operation callback.\n\nThis function calls the following other callbacks and concatenates their results:\n\n  * `c:OpenAPI.Renderer.render_schema_types/2`\n  * `c:OpenAPI.Renderer.render_schema_struct/2`\n  * `c:OpenAPI.Renderer.render_schema_field_function/2`","ref":"OpenAPI.Renderer.Schema.html#render/2"},{"type":"function","title":"OpenAPI.Renderer.Schema.render_field_function/2","doc":"Render a function `__fields__/1` that returns information about schema field types\n\nDefault implementation of `c:OpenAPI.Renderer.render_schema_field_function/2`.\n\nThis implementation renders a function spec and header with one clause for each schema type.","ref":"OpenAPI.Renderer.Schema.html#render_field_function/2"},{"type":"function","title":"OpenAPI.Renderer.Schema.render_struct/2","doc":"Render a single struct definition for all of the given schemas\n\nDefault implementation of `c:OpenAPI.Renderer.render_schema_struct/2`.\n\nThis implementation combines the fields of all schemas to create one struct. It also uses the\n`output.extra_fields` configuration to add additional fields to the struct for private use by\nthe client library. See **Extra Fields** in this module's documentation.","ref":"OpenAPI.Renderer.Schema.html#render_struct/2"},{"type":"function","title":"OpenAPI.Renderer.Schema.render_types/2","doc":"Render the typespec(s) for all of the given schemas\n\nDefault implementation of `c:OpenAPI.Renderer.render_schema_types/2`.\n\nThis implementation uses the `output.extra_fields` configuration to add additional fields to\nthe struct for private use by the client library. See **Extra Fields** in this module's\ndocumentation.","ref":"OpenAPI.Renderer.Schema.html#render_types/2"},{"type":"module","title":"OpenAPI.Renderer.State","doc":"State of the render phase of code generation\n\nThis struct is created at the beginning of the render phase using data from the overall\ngenerator `OpenAPI.State`. It has the following fields:\n\n  * `files`: Map of module names and their associated `t:OpenAPI.Renderer.File.t/0` structs.\n    This map is built using the `operations` and `schemas` data.\n  * `implementation`: Module configured as the implementation of the renderer (defaults to\n    `OpenAPI.Renderer`). Callbacks must use this field when calling other render callbacks.\n  * `operations`: List of `t:OpenAPI.Processor.Operation.t/0` structs processed in the previous\n    phase. These operations may appear in any order due to map key ordering.\n  * `profile`: Name of the active configuration profile. Callbacks must use this field when\n    looking up configuration from the application environment.\n  * `schemas`: Map of schema references to their `t:OpenAPI.Processor.Schema.t/0` structs\n    processed in the previous phase. The reference-keyed map is included to make it easier for\n    callbacks to look up a schema by its reference, as this is often how operations will refer\n    to them.\n\nAll of this state is managed by the code generator, and it is unlikely that a callback would\nneed to transform this struct directly.","ref":"OpenAPI.Renderer.State.html"},{"type":"type","title":"OpenAPI.Renderer.State.t/0","doc":"Render phase state\n\nSee module documentation for additional information.","ref":"OpenAPI.Renderer.State.html#t:t/0"},{"type":"module","title":"OpenAPI.Renderer.Util","doc":"Default implementation and helpers related to formatting and writing files\n\nThis module contains the default implementations for:\n\n  * `c:OpenAPI.Renderer.format/2`\n  * `c:OpenAPI.Renderer.write/2`\n\nIt also contains several helpers for working with ASTs, including the addition of formatting\nmetadata necessary to create consistent code.","ref":"OpenAPI.Renderer.Util.html"},{"type":"function","title":"OpenAPI.Renderer.Util.clean_list/1","doc":"Flatten and remove `nil` elements from a list of AST nodes\n\nThis helper deals with cases in which certain sections of code are rendered conditionally, or\nwith variable lengths of statements. `nil` values will be removed, and nested lists of nodes\nwill be flattened.","ref":"OpenAPI.Renderer.Util.html#clean_list/1"},{"type":"function","title":"Example - OpenAPI.Renderer.Util.clean_list/1","doc":"header = if condition, do: quote(do: IO.puts(\"List\"))\n    statements = for item <- items, do: quote(do: IO.puts(\" * #{item}\"))\n\n    clean_list([header, statements])","ref":"OpenAPI.Renderer.Util.html#clean_list/1-example"},{"type":"function","title":"OpenAPI.Renderer.Util.format/2","doc":"Convert an AST into formatted code as a string\n\nDefault implementation of `c:OpenAPI.Renderer.format/2`.\n\nThe AST may include optional formatting metadata (ex. `delimiter`, `indentation`, or\n`end_of_expression`). It will be formatted to a line width of 98 to match the default Mix\nformatter. In addition, any `@moduledoc` or `@doc` statements that contain a newline character\nwill be modified to use `\"\"\"` as the delimiter.","ref":"OpenAPI.Renderer.Util.html#format/2"},{"type":"function","title":"OpenAPI.Renderer.Util.format_multiline_docs/1","doc":"Walks the given AST and replaces `@doc` and `@moduledoc` strings with `\"\"\"` blocks if the\ncontents have newlines","ref":"OpenAPI.Renderer.Util.html#format_multiline_docs/1"},{"type":"function","title":"OpenAPI.Renderer.Util.put_newlines/1","doc":"Enforce the existence of whitespace after an expression\n\nThis helper is useful for cases in which single-line expressions should be separated from the\nfollowing line by whitespace, but the formatter would not naturally insert that whitespace.\nFor example:\n\n    @my_attribute \"Hello\"\n    @spec my_function :: String.t()\n    def my_function, do: @my_attribute\n\nIt may be desirable to insert whitespace following the module attribute `@my_attribute`. By\ncalling this function on that node, the following will be output:\n\n    @my_attribute \"Hello\"\n\n    @spec my_function :: String.t()\n    def my_function, do: @my_attribute\n\nIf a list of nodes is given, the last node will receive the additional whitespace metadata.","ref":"OpenAPI.Renderer.Util.html#put_newlines/1"},{"type":"function","title":"OpenAPI.Renderer.Util.to_readable_type/2","doc":"Collapse nested unions and replace references with {module, type} identifiers\n\nThis function renders most types exactly as they are expressed internally\n(ex. `{:string, :generic}`), however it transforms certain union types to be more human-readable\nand it replaces schema references with the equivalent `{Module, :type}`.","ref":"OpenAPI.Renderer.Util.html#to_readable_type/2"},{"type":"function","title":"OpenAPI.Renderer.Util.to_type/2","doc":"Render an internal type as a typespec\n\nTo the best of its ability, this function constructs an accurate typespec for the internal\ntype given. Note that this is somewhat lossy; for example, many distinct types of strings will\nmap to the `String.t()` type.","ref":"OpenAPI.Renderer.Util.html#to_type/2"},{"type":"function","title":"OpenAPI.Renderer.Util.unwrap_enums/1","doc":"Replace `enum` types with the equivalent list of `const` types\n\nThis low-level helper is used by `to_type/2` when simplifying union types.","ref":"OpenAPI.Renderer.Util.html#unwrap_enums/1"},{"type":"function","title":"OpenAPI.Renderer.Util.unwrap_unions/1","doc":"Flatten nested union types\n\nThis low-level helper is used by `to_readable_type/2` and `to_type/2` when simplifying union\ntypes.","ref":"OpenAPI.Renderer.Util.html#unwrap_unions/1"},{"type":"function","title":"OpenAPI.Renderer.Util.write/2","doc":"Write a rendered file to the filesystem\n\nDefault implementation of `c:OpenAPI.Renderer.write/2`.\n\nThis implementation writes the file `contents` to the `location` and ensures an additional\nnewline is included at the end of the file. It also ensures that any subdirectories are created\nprior to writing. Any failure will result in a raised error.","ref":"OpenAPI.Renderer.Util.html#write/2"},{"type":"module","title":"OpenAPI.Spec","doc":"OpenAPI description as read by the read phase\n\nModules contained in this namespace represent portions of the OpenAPI spec. While there is some\noccasional processing done by the reader (for example, to add additional context to a schema\nwhile it is read), most of this data is an exact replica of what is contained in the API\ndescription.\n\nAll spec modules have a function `decode/2` which is not intended to be called by client\nlibrary authors, but does the work of parsing the JSON or Yaml contents.","ref":"OpenAPI.Spec.html"},{"type":"type","title":"OpenAPI.Spec.full_path/0","doc":"Fully-qualified absolute file and set of path segments","ref":"OpenAPI.Spec.html#t:full_path/0"},{"type":"type","title":"OpenAPI.Spec.path_segment/0","doc":"Key or index of a Yaml document","ref":"OpenAPI.Spec.html#t:path_segment/0"},{"type":"type","title":"OpenAPI.Spec.ref/0","doc":"Reference to another part of the spec. Reserved for schemas in this library.","ref":"OpenAPI.Spec.html#t:ref/0"},{"type":"type","title":"OpenAPI.Spec.t/0","doc":"Open API specification","ref":"OpenAPI.Spec.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Components","doc":"Raw components map from the OpenAPI spec","ref":"OpenAPI.Spec.Components.html"},{"type":"type","title":"OpenAPI.Spec.Components.t/0","doc":"","ref":"OpenAPI.Spec.Components.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.ExternalDocumentation","doc":"Raw external documentation definition from the OpenAPI spec","ref":"OpenAPI.Spec.ExternalDocumentation.html"},{"type":"type","title":"OpenAPI.Spec.ExternalDocumentation.t/0","doc":"","ref":"OpenAPI.Spec.ExternalDocumentation.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Info","doc":"Raw information map from the OpenAPI spec","ref":"OpenAPI.Spec.Info.html"},{"type":"type","title":"OpenAPI.Spec.Info.t/0","doc":"","ref":"OpenAPI.Spec.Info.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Info.Contact","doc":"Raw contact information from the OpenAPI spec","ref":"OpenAPI.Spec.Info.Contact.html"},{"type":"type","title":"OpenAPI.Spec.Info.Contact.t/0","doc":"","ref":"OpenAPI.Spec.Info.Contact.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Info.License","doc":"Raw license information from the OpenAPI spec","ref":"OpenAPI.Spec.Info.License.html"},{"type":"type","title":"OpenAPI.Spec.Info.License.t/0","doc":"","ref":"OpenAPI.Spec.Info.License.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Link","doc":"Raw link from the OpenAPI spec","ref":"OpenAPI.Spec.Link.html"},{"type":"type","title":"OpenAPI.Spec.Link.t/0","doc":"","ref":"OpenAPI.Spec.Link.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Path.Header","doc":"Raw path header from the OpenAPI spec","ref":"OpenAPI.Spec.Path.Header.html"},{"type":"type","title":"OpenAPI.Spec.Path.Header.t/0","doc":"","ref":"OpenAPI.Spec.Path.Header.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Path.Item","doc":"Raw operation path from the OpenAPI spec","ref":"OpenAPI.Spec.Path.Item.html"},{"type":"function","title":"OpenAPI.Spec.Path.Item.decode_delete/2","doc":"","ref":"OpenAPI.Spec.Path.Item.html#decode_delete/2"},{"type":"function","title":"OpenAPI.Spec.Path.Item.decode_get/2","doc":"","ref":"OpenAPI.Spec.Path.Item.html#decode_get/2"},{"type":"function","title":"OpenAPI.Spec.Path.Item.decode_head/2","doc":"","ref":"OpenAPI.Spec.Path.Item.html#decode_head/2"},{"type":"function","title":"OpenAPI.Spec.Path.Item.decode_options/2","doc":"","ref":"OpenAPI.Spec.Path.Item.html#decode_options/2"},{"type":"function","title":"OpenAPI.Spec.Path.Item.decode_parameters/2","doc":"","ref":"OpenAPI.Spec.Path.Item.html#decode_parameters/2"},{"type":"function","title":"OpenAPI.Spec.Path.Item.decode_patch/2","doc":"","ref":"OpenAPI.Spec.Path.Item.html#decode_patch/2"},{"type":"function","title":"OpenAPI.Spec.Path.Item.decode_post/2","doc":"","ref":"OpenAPI.Spec.Path.Item.html#decode_post/2"},{"type":"function","title":"OpenAPI.Spec.Path.Item.decode_put/2","doc":"","ref":"OpenAPI.Spec.Path.Item.html#decode_put/2"},{"type":"function","title":"OpenAPI.Spec.Path.Item.decode_trace/2","doc":"","ref":"OpenAPI.Spec.Path.Item.html#decode_trace/2"},{"type":"type","title":"OpenAPI.Spec.Path.Item.t/0","doc":"","ref":"OpenAPI.Spec.Path.Item.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Path.Operation","doc":"Raw operation from the OpenAPI spec","ref":"OpenAPI.Spec.Path.Operation.html"},{"type":"function","title":"OpenAPI.Spec.Path.Operation.decode_parameters/2","doc":"","ref":"OpenAPI.Spec.Path.Operation.html#decode_parameters/2"},{"type":"function","title":"OpenAPI.Spec.Path.Operation.decode_request_body/2","doc":"","ref":"OpenAPI.Spec.Path.Operation.html#decode_request_body/2"},{"type":"type","title":"OpenAPI.Spec.Path.Operation.t/0","doc":"","ref":"OpenAPI.Spec.Path.Operation.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Path.Parameter","doc":"Raw path parameter from the OpenAPI spec","ref":"OpenAPI.Spec.Path.Parameter.html"},{"type":"function","title":"OpenAPI.Spec.Path.Parameter.decode_content/2","doc":"","ref":"OpenAPI.Spec.Path.Parameter.html#decode_content/2"},{"type":"function","title":"OpenAPI.Spec.Path.Parameter.decode_examples/2","doc":"","ref":"OpenAPI.Spec.Path.Parameter.html#decode_examples/2"},{"type":"function","title":"OpenAPI.Spec.Path.Parameter.decode_schema/2","doc":"","ref":"OpenAPI.Spec.Path.Parameter.html#decode_schema/2"},{"type":"type","title":"OpenAPI.Spec.Path.Parameter.t/0","doc":"","ref":"OpenAPI.Spec.Path.Parameter.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.RequestBody","doc":"Raw request body from the OpenAPI spec","ref":"OpenAPI.Spec.RequestBody.html"},{"type":"type","title":"OpenAPI.Spec.RequestBody.t/0","doc":"","ref":"OpenAPI.Spec.RequestBody.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Response","doc":"Raw response from the OpenAPI spec","ref":"OpenAPI.Spec.Response.html"},{"type":"type","title":"OpenAPI.Spec.Response.t/0","doc":"","ref":"OpenAPI.Spec.Response.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Schema","doc":"Raw JSON schema specification from the OpenAPI spec","ref":"OpenAPI.Spec.Schema.html"},{"type":"type","title":"OpenAPI.Spec.Schema.t/0","doc":"JSON Schema specification\n\nIn addition to the data derived from the JSON or Yaml specification, this struct also contains\nsome contextual information including its absolute and referenced location in the spec. All\nsuch fields are prefixed with `$oag_`.","ref":"OpenAPI.Spec.Schema.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Schema.Discriminator","doc":"Raw schema discriminator from the OpenAPI spec","ref":"OpenAPI.Spec.Schema.Discriminator.html"},{"type":"type","title":"OpenAPI.Spec.Schema.Discriminator.t/0","doc":"","ref":"OpenAPI.Spec.Schema.Discriminator.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Schema.Encoding","doc":"Raw schema encoding from the OpenAPI spec","ref":"OpenAPI.Spec.Schema.Encoding.html"},{"type":"type","title":"OpenAPI.Spec.Schema.Encoding.t/0","doc":"","ref":"OpenAPI.Spec.Schema.Encoding.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Schema.Example","doc":"Raw schema example from the OpenAPI spec","ref":"OpenAPI.Spec.Schema.Example.html"},{"type":"type","title":"OpenAPI.Spec.Schema.Example.t/0","doc":"","ref":"OpenAPI.Spec.Schema.Example.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Schema.Media","doc":"Raw media schema from the OpenAPI spec","ref":"OpenAPI.Spec.Schema.Media.html"},{"type":"type","title":"OpenAPI.Spec.Schema.Media.t/0","doc":"","ref":"OpenAPI.Spec.Schema.Media.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Schema.XML","doc":"Raw XML definition from the OpenAPI spec","ref":"OpenAPI.Spec.Schema.XML.html"},{"type":"type","title":"OpenAPI.Spec.Schema.XML.t/0","doc":"","ref":"OpenAPI.Spec.Schema.XML.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Server","doc":"Raw server from the OpenAPI spec","ref":"OpenAPI.Spec.Server.html"},{"type":"type","title":"OpenAPI.Spec.Server.t/0","doc":"","ref":"OpenAPI.Spec.Server.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Server.Variable","doc":"Raw server variable from the OpenAPI spec","ref":"OpenAPI.Spec.Server.Variable.html"},{"type":"type","title":"OpenAPI.Spec.Server.Variable.t/0","doc":"","ref":"OpenAPI.Spec.Server.Variable.html#t:t/0"},{"type":"module","title":"OpenAPI.Spec.Tag","doc":"Raw tag from the OpenAPI spec","ref":"OpenAPI.Spec.Tag.html"},{"type":"type","title":"OpenAPI.Spec.Tag.t/0","doc":"","ref":"OpenAPI.Spec.Tag.html#t:t/0"},{"type":"extras","title":"Overview","doc":"# OpenAPI Generator for Elixir\n\n[![Hex.pm](https://img.shields.io/hexpm/v/oapi_generator)](https://hex.pm/packages/oapi_generator)\n[![Documentation](https://img.shields.io/badge/hex-docs-blue)](https://hexdocs.pm/oapi_generator)\n[![Contributor Covenant](https://img.shields.io/badge/Contributor%20Covenant-2.1-4baaaa.svg)](CODE_OF_CONDUCT.md)\n\n_A highly-configurable code generator that combines ergonomics with maintainability._\n\n---\n\n[OpenAPI](https://swagger.io/specification/) is a standard way to describe REST APIs on the web.\nAnyone can create an OpenAPI description document that includes the available endpoints, expected request data, and possible responses.\nFor example, [GitHub](https://github.com/github/rest-api-description) maintains a comprehensive OpenAPI description for their services.\n\nGenerating code from an OpenAPI description can be relatively easy — this project [certainly isn't the first](https://openapi-generator.tech/docs/generators/elixir) — but there's a catch: API descriptions often don't translate into ergonomic code.\nMost users of an API client library don't want to think about the difference between a `NullableRepository` and `Repository`, as in the OpenAPI 3.0 GitHub API description.\n(They have the same fields, but one has `nullable: true`.)\nUsers just want to get back a `%Repository{}` or `nil`.\n\nThe goal of this library is to create **ergonomic** client libraries from OpenAPI descriptions.\nAt the same time, the changes made to the code are easily repeatable (fully automated) for the sake of maintainability.\nThink: the friendliness of your favorite hand-crafted client library applied to the scale of large APIs.\n\n> _See an example client library [here](https://github.com/aj-foster/open-api-github)._\n\nFor more on how this is accomplished, see **Configuration** below and the [configuration guide](guides/configuration.md).","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"This library is available on Hex.pm.\nAdd the dependency in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:oapi_generator, \"~> 0.2.0\", only: :dev, runtime: false}\n  ]\nend\n```\n\nThen install the dependency using `mix deps.get`.\nMost libraries only need access to the `mix api.gen` task in a development environment.\nIf your use case requires calling the generator in production or testing, be sure to modify or remove `only: :dev` and `runtime: false` as appropriate.","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - Overview","doc":"The real power of this library is in the available configuration.\nAlthough the options are conceptually simple, they add up to a better user experience.\n\nThis project uses configuration profiles to allow multiple configurations with the same package.\nTo get started, create a profile called `default` in your configuration file:\n\n```elixir\n# config/config.exs\n\nimport Config\n\nconfig :oapi_generator, default: [\n  output: [\n    base_module: Example,\n    location: \"lib/example\"\n  ]\n]\n```\n\nThis is the minimum viable configuration for most client libraries.\nIt will create modules namespaced with `Example.` and save files in `lib/example`.\n\nSome the options supported by the generator out-of-the-box include:\n\n* Ignoring schemas and operations\n* Renaming schemas\n* Grouping schemas into module namespaces\n* Merging schemas to create a struct with multiple typespecs\n* Writing schemas and operation modules to different locations\n* Introducing additional schema fields\n* Adding custom `use` statements to generated modules\n* Overriding function return types\n\nFor more information, see the [configuration guide](guides/configuration.md).","ref":"readme.html#configuration"},{"type":"extras","title":"Plugins - Overview","doc":"If the available configuration isn't enough, client library authors can also reimplement portions of the code generator using _plugins_.\nMost of the crucial parts of the processing and rendering of code are implemented as default callbacks for a behaviour.\nThese can be overridden for additional flexibility.\nSee the [plugins guide](guides/plugins.md) for additional information.","ref":"readme.html#plugins"},{"type":"extras","title":"Usage - Overview","doc":"Once the library is installed and configured, use `mix api.gen` with the name of the configuration profile and the OpenAPI description file(s):\n\n```shell\nmix api.gen default path/to/rest-api-description/spec.yaml\n```","ref":"readme.html#usage"},{"type":"extras","title":"Further Reading - Overview","doc":"* [Code of Conduct](CODE_OF_CONDUCT.md)\n* [Contribution Guidelines](CONTRIBUTING.md)\n* [License](LICENSE)\n* [OpenAPI Specification](https://spec.openapis.org/oas/latest.html)","ref":"readme.html#further-reading"},{"type":"extras","title":"Sponsorship - Overview","doc":"If you like this library or it makes you money, please consider [sponsoring](https://github.com/sponsors/aj-foster).","ref":"readme.html#sponsorship"},{"type":"extras","title":"Code of Conduct","doc":"# Contributor Covenant Code of Conduct","ref":"code_of_conduct.html"},{"type":"extras","title":"Our Pledge - Code of Conduct","doc":"We as members, contributors, and leaders pledge to make participation in our\ncommunity a harassment-free experience for everyone, regardless of age, body\nsize, visible or invisible disability, ethnicity, sex characteristics, gender\nidentity and expression, level of experience, education, socio-economic status,\nnationality, personal appearance, race, caste, color, religion, or sexual\nidentity and orientation.\n\nWe pledge to act and interact in ways that contribute to an open, welcoming,\ndiverse, inclusive, and healthy community.","ref":"code_of_conduct.html#our-pledge"},{"type":"extras","title":"Our Standards - Code of Conduct","doc":"Examples of behavior that contributes to a positive environment for our\ncommunity include:\n\n* Demonstrating empathy and kindness toward other people\n* Being respectful of differing opinions, viewpoints, and experiences\n* Giving and gracefully accepting constructive feedback\n* Accepting responsibility and apologizing to those affected by our mistakes,\n  and learning from the experience\n* Focusing on what is best not just for us as individuals, but for the overall\n  community\n\nExamples of unacceptable behavior include:\n\n* The use of sexualized language or imagery, and sexual attention or advances of\n  any kind\n* Trolling, insulting or derogatory comments, and personal or political attacks\n* Public or private harassment\n* Publishing others' private information, such as a physical or email address,\n  without their explicit permission\n* Other conduct which could reasonably be considered inappropriate in a\n  professional setting","ref":"code_of_conduct.html#our-standards"},{"type":"extras","title":"Enforcement Responsibilities - Code of Conduct","doc":"Community leaders are responsible for clarifying and enforcing our standards of\nacceptable behavior and will take appropriate and fair corrective action in\nresponse to any behavior that they deem inappropriate, threatening, offensive,\nor harmful.\n\nCommunity leaders have the right and responsibility to remove, edit, or reject\ncomments, commits, code, wiki edits, issues, and other contributions that are\nnot aligned to this Code of Conduct, and will communicate reasons for moderation\ndecisions when appropriate.","ref":"code_of_conduct.html#enforcement-responsibilities"},{"type":"extras","title":"Scope - Code of Conduct","doc":"This Code of Conduct applies within all community spaces, and also applies when\nan individual is officially representing the community in public spaces.\nExamples of representing our community include using an official e-mail address,\nposting via an official social media account, or acting as an appointed\nrepresentative at an online or offline event.","ref":"code_of_conduct.html#scope"},{"type":"extras","title":"Enforcement - Code of Conduct","doc":"Instances of abusive, harassing, or otherwise unacceptable behavior may be\nreported to the community leaders responsible for enforcement\n[by email](mailto:public@aj-foster.com).\nAll complaints will be reviewed and investigated promptly and fairly.\n\nAll community leaders are obligated to respect the privacy and security of the\nreporter of any incident.","ref":"code_of_conduct.html#enforcement"},{"type":"extras","title":"Enforcement Guidelines - Code of Conduct","doc":"Community leaders will follow these Community Impact Guidelines in determining\nthe consequences for any action they deem in violation of this Code of Conduct:","ref":"code_of_conduct.html#enforcement-guidelines"},{"type":"extras","title":"1. Correction - Code of Conduct","doc":"**Community Impact**: Use of inappropriate language or other behavior deemed\nunprofessional or unwelcome in the community.\n\n**Consequence**: A private, written warning from community leaders, providing\nclarity around the nature of the violation and an explanation of why the\nbehavior was inappropriate. A public apology may be requested.","ref":"code_of_conduct.html#1-correction"},{"type":"extras","title":"2. Warning - Code of Conduct","doc":"**Community Impact**: A violation through a single incident or series of\nactions.\n\n**Consequence**: A warning with consequences for continued behavior. No\ninteraction with the people involved, including unsolicited interaction with\nthose enforcing the Code of Conduct, for a specified period of time. This\nincludes avoiding interactions in community spaces as well as external channels\nlike social media. Violating these terms may lead to a temporary or permanent\nban.","ref":"code_of_conduct.html#2-warning"},{"type":"extras","title":"3. Temporary Ban - Code of Conduct","doc":"**Community Impact**: A serious violation of community standards, including\nsustained inappropriate behavior.\n\n**Consequence**: A temporary ban from any sort of interaction or public\ncommunication with the community for a specified period of time. No public or\nprivate interaction with the people involved, including unsolicited interaction\nwith those enforcing the Code of Conduct, is allowed during this period.\nViolating these terms may lead to a permanent ban.","ref":"code_of_conduct.html#3-temporary-ban"},{"type":"extras","title":"4. Permanent Ban - Code of Conduct","doc":"**Community Impact**: Demonstrating a pattern of violation of community\nstandards, including sustained inappropriate behavior, harassment of an\nindividual, or aggression toward or disparagement of classes of individuals.\n\n**Consequence**: A permanent ban from any sort of public interaction within the\ncommunity.","ref":"code_of_conduct.html#4-permanent-ban"},{"type":"extras","title":"Attribution - Code of Conduct","doc":"This Code of Conduct is adapted from the [Contributor Covenant][homepage],\nversion 2.1, available at\n[https://www.contributor-covenant.org/version/2/1/code_of_conduct.html][v2.1].\n\nCommunity Impact Guidelines were inspired by\n[Mozilla's code of conduct enforcement ladder][Mozilla CoC].\n\nFor answers to common questions about this code of conduct, see the FAQ at\n[https://www.contributor-covenant.org/faq][FAQ]. Translations are available at\n[https://www.contributor-covenant.org/translations][translations].\n\n[homepage]: https://www.contributor-covenant.org\n[v2.1]: https://www.contributor-covenant.org/version/2/1/code_of_conduct.html\n[Mozilla CoC]: https://github.com/mozilla/diversity\n[FAQ]: https://www.contributor-covenant.org/faq\n[translations]: https://www.contributor-covenant.org/translations","ref":"code_of_conduct.html#attribution"},{"type":"extras","title":"Contributing","doc":"# Contribution Guidelines\n\nHello there.\nThank you for your interest in contributing to this project.\nWith a wide variety of OpenAPI descriptions in the world, this project could use your assistance.\n\nReading and following the guidelines in this document is an act of kindness and respect for other contributors.\nWith your help, we can address issues, make changes, and work together efficiently.","ref":"contributing.html"},{"type":"extras","title":"Ways to Contribute - Contributing","doc":"There are many ways to contribute to this project:\n\n* Individuals with experience in Elixir can help us to address observed and potential issues.\n* Users of REST APIs with OpenAPI descriptions can try the generator and report any issues or recommendations for improvement they encounter.\n* Users of the package can help to test new use-cases and patterns. Documentation improves everyone's experience.\n* Anyone can assist in the triage of bugs, identifying root causes, and proposing solutions.\n\nPlease keep in mind the intended scope of this package: to provide a code generator that balances an ergonomic user experience with maintainability.\nAssume that the OpenAPI description that serves as input to this library will change often, and manually changing the output is impractical.","ref":"contributing.html#ways-to-contribute"},{"type":"extras","title":"Ground Rules - Contributing","doc":"All contributions to this project must align with the [code of conduct](CODE_OF_CONDUCT.md).\nBeyond that, we ask:\n\n* Please be kind. Maintaining this project is not paid work.\n* Please create an issue before embarking on major refactors or new features.\n* Let's make reasonable effort to support as many OpenAPI descriptions as possible.","ref":"contributing.html#ground-rules"},{"type":"extras","title":"Security Issues - Contributing","doc":"If you find a security-related issue with this project, please refrain from opening a public issue and instead [email the maintainer](mailto:public@aj-foster.com).","ref":"contributing.html#security-issues"},{"type":"extras","title":"Releases - Contributing","doc":"For maintainers, the process of releasing this package to [Hex.pm](https://hex.pm/packages/oapi_generator) centers around git tags.\nTo make a new release:\n\n1. Update the Changelog with a new header that has today's date and the new version.\n  Include any missing notes from changes since the last release, and any additional upgrade instructions users may need.\n2. Update the `@version` number in `mix.exs`.\n  The form should be `X.Y.Z`, with optional suffixes, but no leading `v`.\n3. Update the **Installation** instructions in `README.md` to have the newest non-suffixed version number.\n4. Commit the above changes with a generic commit message, such as `Release X.Y.Z`.\n5. Tag the commit as `X.Y.Z`, with optional suffixes, but no leading `v`.\n6. Push the commits and tag (for example, `git push origin main --tags`).\n7. Observe the GitHub Action titled **Release**.\n  This action automatically publishes the package to Hex.pm.","ref":"contributing.html#releases"},{"type":"extras","title":"License","doc":"MIT License\n\nCopyright (c) 2022 AJ Foster\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"Configuration","doc":"# Configuration\n\nThis project exists for its configuration capabilities.\nMost API client libraries are **either** feature complete (due to code generation) **or** ergonomic (due to hand-crafting code), but not both.\nWith the right configuration — and a little bit of luck with the OpenAPI description — your client library can be both.\n\nThe following sections outline the configuration available for the _default implementations_ of the various callbacks used by this library.\nSee the [Plugins](plugins.md) documentation for information about overriding the default behaviours with custom implementations.","ref":"configuration.html"},{"type":"extras","title":"Profiles - Configuration","doc":"This library uses configuration _profiles_ to allow for multiple sets of configuration at once.\nIn practice, this means that all configuration lives under a top-level key that will be referenced when running the generator.\nFor example:\n\n```elixir\n# config/config.exs\n\nimport Config\n\nconfig :oapi_generator,\n  my_profile: [\n    output: [\n      base_module: MyClientLibrary\n    ]\n  ]\n```\n\nTo use this configuration, run `mix api.gen my_profile path/to/description.json` or call `OpenAPI.run(\"my_profile\", [\"path/to/description/json\"])`.\n\n> Not sure what to name the profile? Start with `default`.\n\n---","ref":"configuration.html#profiles"},{"type":"extras","title":"Operations - Configuration","doc":"Operations are the actions you can take in an API: the HTTP requests that are possible.\nThis code generator takes an operation-first mindset when generating code.\nFor example, schemas that are defined in a description but not referenced in any API operation will not be output.\n\nFollowing is an example operation module that has been annotated with the configuration options involved in various parts of the rendered output.","ref":"configuration.html#operations"},{"type":"extras","title":"Example Module - Configuration","doc":"```elixir\n# Existence: `ignore`\n# Location: `output.location`, `output.operation_subdirectory`\n# Module name: `output.base_module`, `naming.default_operation_module`\ndefmodule Example.Operations do\n  @moduledoc \"\"\"\n  Provides API endpoints related to repos\n  \"\"\"\n  # `output.operation_use`, `output_schema_use` (if schemas are present)\n  use Example.Helper\n\n  # `output.default_client`\n  @default_client Example.Client\n\n  # Schemas would appear here, if present.\n\n  @doc \"\"\"\n  Summary of the operation\n  \"\"\"\n  # `output.base_module`, `output.types.error`\n  @spec my_operation(String.t(), map, keyword) :: :ok | {:error, Example.Error.t()}\n  def my_operation(path_param, body, opts \\\\ []) do\n    client = opts[:client] || @default_client\n    query = Keyword.take(opts, [:query_param])\n\n    client.request(%{\n      args: [path_param: path_param, body: body],\n      call: {Example.Operations, :my_operation},\n      url: \"/path/to/#{path_param}\",\n      body: body,\n      method: :post,\n      query: query,\n      request: [{\"application/json\", :map}],\n      response: [{200, :map}, {404, {Example.NotFoundError, :t}}],\n      opts: opts\n    })\n  end\nend\n```\n\nBelow are some of the highlights from the available configuration.\nSee the **Option Reference** section for an exhaustive list of the available options.","ref":"configuration.html#example-module"},{"type":"extras","title":"Processor: Naming - Configuration","doc":"At the moment, configuration related to the naming and placement of operation functions is limited.\nIf you have ideas related to this, please share them in GitHub Discussions.\n\nIf an operation does not have any tags, and its operation ID does not contain any `/` characters, it will be placed in a default module named `[BaseModule].Operations`.\nThis module name can be controlled using the `naming.default_operation_module` configuration:\n\n```elixir\nconfig :oapi_generator, default: [\n  naming: [\n    base_module: MyClientLibrary,\n    default_operation_module: MyOperations\n  ]\n]\n```\n\nLearn more in the documentation for `OpenAPI.Processor.Naming.operation_modules/2`.","ref":"configuration.html#processor-naming"},{"type":"extras","title":"Processor: Ignore - Configuration","doc":"The `ignore` option allows operations to be ignored based on their Operation IDs or paths in the API description.\nPatterns can be exact strings or regular expressions.\n\nLearn more in the documentation for `OpenAPI.Processor.Ignore`.","ref":"configuration.html#processor-ignore"},{"type":"extras","title":"Renderer: Default Client - Configuration","doc":"`output.default_client` defines the module that will be called in each operation function.\nBy allowing the client module to be defined at the time of code generation (with this option) and at runtime (with the `client` option passed to any operation call), this code generator maximizes flexibility.\nFor example, tests can supply a special test client module when they call client operations.\n\nThis need not be the end of the flexibility, however.\nThe GitHub client library defines a \"stack\" system within its client module, allowing users to easily swap libraries for handling HTTP requests and decoding JSON.","ref":"configuration.html#renderer-default-client"},{"type":"extras","title":"Renderer: Meta-programming - Configuration","doc":"`output.operation_use` defines a module that should be included as a `use [output.operation_use]` statement at the top of every module that contains operations.\nThis can be useful for meta-programming, such as using compilation hooks to performed advanced modifications of the generated code.","ref":"configuration.html#renderer-meta-programming"},{"type":"extras","title":"Renderer: Types - Configuration","doc":"`output.types`, and specifically its `error` key, allows library authors to override the return type of operation functions.\nIt's fairly common for client libraries to create their own error that acts as a superset for all of the errors defined in the OpenAPI description.\nThis provides a consistent interface for client library consumers.\n\n---","ref":"configuration.html#renderer-types"},{"type":"extras","title":"Schemas - Configuration","doc":"Much of the configuration of this code generator relates to the naming and output of schemas.\nRemember that the library takes an operation-first mindset, and will not output schemas that are not actively used by an operation.\n\nFollowing is an example operation module that has been annotated with the configuration options involved in various parts of the rendered output.","ref":"configuration.html#schemas"},{"type":"extras","title":"Example Schema - Configuration","doc":"```elixir\n# Existence: `ignore`\n# Location: `output.location`, `output.schema_subdirectory`\n# Module name: `output.base_module`\ndefmodule Example.MySchema do\n  @moduledoc \"\"\"\n  Provides struct and types for a MySchema\n  \"\"\"\n  # `output_schema_use`\n  use Example.Helper\n\n  @type t :: %__MODULE__{\n    example: String.t()\n  }\n\n  defstruct [\n    # output.extra_fields\n    :__extra__,\n    :example\n  ]\n\n  @doc false\n  @spec __fields__(atom) :: keyword\n  def __fields__(type \\\\ :t)\n\n  def __fields__(:t) do\n    [\n      example: {:string, :generic}\n    ]\n  end\nend\n```\n\nBelow are some of the highlights from the available configuration.\nSee the **Option Reference** section for an exhaustive list of the available options.","ref":"configuration.html#example-schema"},{"type":"extras","title":"Processor: Naming - Configuration","doc":"Schemas can be renamed, grouped, and merged.\nApplying variations of these options to the API description can turn generated code from confusing to ergonomic rather quickly.\n\nLearn more in the documentation for `OpenAPI.Processor.Naming.schema_module_and_type/2`.","ref":"configuration.html#processor-naming"},{"type":"extras","title":"Processor: Ignore - Configuration","doc":"The `ignore` option allows operations to be ignored based on their Operation IDs or paths in the API description.\nPatterns can be exact strings or regular expressions.\n\nLearn more in the documentation for `OpenAPI.Processor.Ignore`.","ref":"configuration.html#processor-ignore"},{"type":"extras","title":"Renderer: Extra Fields - Configuration","doc":"`output.extra_fields` allows authors to insert additional fields into every outputted schema.\nFor example, the GitHub library adds an `__info__` map to each schema.\nLike the `__meta__` field provided by `Ecto.Schema` or the `private` field on `Plug.Conn`, this field can hold additional library-specific information without interfering with the API's own data.","ref":"configuration.html#renderer-extra-fields"},{"type":"extras","title":"Renderer: Meta-programming - Configuration","doc":"`output.schema_use` allows authors to insert a `use [output.schema_use]` statement at the top of every module containing a schema.\nA corresponding module must be supplied by the library with a `__using__/1` macro.\nInside this macro, library authors can inject additional code that may be helpful.\nFor example:\n\n```elixir\ndefmodule MyLibrary.Schema do\n  defmacro __using__(_opts) do\n    quote do\n      @derive {Jason.Encoder, except: [:__info__]}\n    end\n  end\nend\n```\n\nInside the macro, you can reference the name of the current module using `__MODULE__`, along with any other variables that are part of the macro environment.\n\n---","ref":"configuration.html#renderer-meta-programming"},{"type":"extras","title":"Option Reference - Configuration","doc":"Below are all of the configuration options offered by the default implementations of the processor and renderer.\nThey are expressed in terms of nested keyword lists.\nFor example, `output.base_module` represents the following:\n\n```elixir\nconfig :oapi_generator, default: [\n  output [\n    base_module: ...\n  ]\n]\n```\n\nRemember that all configuration values must be contained within a profile.\n\n* `ignore`: List of patterns (string or regular expression) representing operations or schemas to ignore during code generation.\n  Defaults to an empty list of patterns.\n  See `OpenAPI.Processor.Ignore` for more information.\n\n* `naming.default_operation_module`: Module into which operation functions will be rendered if a module cannot be determined based on their operation ID or tags.\n  The configuration should not include the base module configured in `output.base_module`.\n  Defaults to `Operations`.\n  See `OpenAPI.Processor.Naming.operation_modules/2` for more information.\n\n* `naming.field_casing`: Either `:camel`, `:snake`, or `nil` (default) to output schema field names as `camelCase`, `snake_case`, or leave the fields name as-is from the API description.\n  Changing the field casing is likely to be a breaking change for clients, unless the API description consistently uses the same casing.\n  Setting this field may be necessary if field names require normalization (ex. if a field begins with a number).\n\n* `naming.group`: List of module namespaces to use while naming operations and schemas.\n  Defaults to an empty list of modules.\n  See `OpenAPI.Processor.Naming.group_schema/2` for more information.\n\n* `naming.merge`: List of tuples containing patterns and replacements to use when merging schema modules.\n  Defaults to an empty list of merges.\n  See `OpenAPI.Processor.Naming.merge_schema/2` for more information.\n\n* `naming.operation_use_tags`: Whether to use tags when determining module names for operations.\n  Defaults to `true`.\n  See `OpenAPI.Processor.Naming.operation_modules/2` for more information.\n\n* `naming.rename`: List of tuples containing patterns and replacements to use when renaming modules.\n  Defaults to an empty list of replacements.\n  See `OpenAPI.Processor.Naming.rename_schema/2` for more information.\n\n* `output.base_module`: Base module of the generated code.\n  This is often the name of the library you intend to create, and it acts as a prefix for all rendered modules.\n  Defaults to `nil`, or no module prefix.\n  See `OpenAPI.Renderer.Module.render/2` for more information.\n\n* `output.default_client`: Default client to use in operation modules.\n  Every operation function calls a dynamic module to perform requests, and this configuration determines the default value.\n  Defaults to `[output.base_module].Client`.\n  See `OpenAPI.Renderer.Module.render_default_client/2` for more information.\n\n* `output.extra_fields`: Additional fields to add to all schema struct definitions and their typespecs, expressed as a keyword list of types.\n  This can be useful for private data the library wishes to add to all outputted data.\n  Defaults to an empty list of fields.\n  See `OpenAPI.Renderer.Schema` for more information.\n\n* `output.location`: Base filesystem location for all rendered files.\n  This is often a directory like `lib` to follow Elixir conventions.\n  Defaults to the current working directory.\n  See `OpenAPI.Renderer.Module.filename/2` for more information.\n\n* `output.operation_call.request`: Format of the `request` key in the body of an operation function.\n  By default, the possible request body schemas are formatted as a `:list` of tuples with the content type and the schema.\n  A value of `:map` will cause a map to be output instead.\n  See `OpenAPI.Renderer.Operation.render_function/2` for more information.\n\n* `output.operation_subdirectory`: Subdirectory to use for all rendered files containing operations.\n  This is an optional way to colocate generated operation modules in a single directory away from other parts of the client library.\n  Defaults to the same location as `output.location`.\n  See `OpenAPI.Renderer.Module.filename/2` for more information.\n\n* `output.operation_use`: Module to include as a `use` statement at the top of every module containing operations.\n  Defaults to no used module.\n  See `OpenAPI.Renderer.Module.render_using/2` for more information.\n\n* `output.schema_subdirectory`: Subdirectory to use for all rendered files containing only schemas.\n  This is an optional way to colocate generated schema modules in a single directory away from other parts of the client library.\n  Defaults to the same location as `output.location`.\n  See `OpenAPI.Renderer.Module.filename/2` for more information.\n\n* `output.schema_use`: Module to include as a `use` statement at the top of every module containing only schemas.\n  Defaults to no used module.\n  See `OpenAPI.Renderer.Module.render_using/2` for more information.\n\n* `output.types.error`: Type to override the error return type of all operation functions.\n  This is useful when the client intends to normalize errors from the API.\n  By default, the error type is a union of all possible error responses for the operation.\n  See `OpenAPI.Renderer.Operation.render_spec/2` for more information.\n\n* `reader.additional_files`: List of paths to supplemental root files of an API description.\n  Defaults to an empty list.\n  See `OpenAPI.Reader` for more information.\n\n* `reader.file`: Path to the root file of an API description.\n  This option is often supplied to the `mix api.gen` mix task instead.\n  See `OpenAPI.Reader` for more information.","ref":"configuration.html#option-reference"},{"type":"extras","title":"Plugins","doc":"# Plugins\n\nSometimes, the configuration available using the default implementation of this library isn't enough.\nFor advanced use-cases, library authors have the option to override the default implementations and create their own workflows.\nThis guide provides more details and a brief example of such an integration.\n\nFirst, however, it is important to understand how the library is divided up:","ref":"plugins.html"},{"type":"extras","title":"Phases - Plugins","doc":"There are three overall phases involved in code generation:\n\n1. **Read** the API description, dereference it, and create a large in-memory data structure representing the specification.\n  This phase begins with one or more filenames (JSON or YAML) to read and ends with an Elixir representation of each document.\n2. **Process** the description, converting the data into structures that are more useful for code generation.\n  This phase begins with the dereferenced API description and ends with a modified set of structs that represent the files to be written.\n3. **Render** the code using an AST-based renderer.\n  This phase begins with the final state of the API description and ends with formatted Elixir code written to the filesystem.\n\nThrough all three phases, the library uses an `OpenAPI.State` struct to keep track of changes.\nEach phase also implements its own \"state\" struct, with intermediate data useful for that phase.","ref":"plugins.html#phases"},{"type":"extras","title":"Behaviours - Plugins","doc":"Both the _process_ and _render_ phases define a behaviour with optional callbacks.\nBy default, the following modules are used for each phase:\n\n```elixir\nconfig :oapi_generator,\n  my_profile: [\n    processor: OpenAPI.Processor,\n    renderer: OpenAPI.Renderer\n  ]\n```\n\nClient library authors may configure their own module in place of the default implementation.\nFor example, a custom renderer may start like this:\n\n```elixir\n# config/config.exs\n\nconfig :oapi_generator,\n  my_profile: [\n    renderer: MyLibrary.Renderer\n  ]\n```\n\n```elixir\n# lib/my_library/renderer.ex\n\ndefmodule MyLibrary.Renderer do\n  use OpenAPI.Renderer\nend\n```\n\nWith `use OpenAPI.Renderer` (or similarly with `use OpenAPI.Processor`), default implementations of all behaviour callbacks are provided automatically.\nThis allows authors to override only those callbacks they are interested in.\n\n```elixir\n# lib/my_library/renderer.ex\n\ndefmodule MyLibrary.Renderer do\n  use OpenAPI.Renderer\n\n  # Override the default write/2 callback\n  # with an alternative implementation.\n  def write(state, file) do\n    # ...\n  end\nend\n```\n\nThe full list of callbacks can be found in the [**Callbacks**](#callbacks) section below and in the documentation for `OpenAPI.Processor` and `OpenAPI.Renderer`.","ref":"plugins.html#behaviours"},{"type":"extras","title":"Configuration - Plugins","doc":"Since custom plugins are implemented in each client applications, authors can choose to configure them in whatever manner suits the client application best.\nIt is recommended that the method of configuration maximize the repeatability of the code generation for easier contributions.\n\nDefault implementations of each callback generally use the application environment for configuration.\nBecause of this, if library authors use the same configuration profiles setup as the default implementations, it is recommended that each custom plugin choose a unique configuration key within the profile that does not conflict with other callbacks.","ref":"plugins.html#configuration"},{"type":"extras","title":"Callbacks - Plugins","doc":"Below is a list of all available callbacks in order of their use, along with their default implementations:\n\n**Processor**: `OpenAPI.Processor`\n\n* For each operation\n  * `c:OpenAPI.Processor.ignore_operation?/2`: `OpenAPI.Processor.Ignore.ignore_operation?/2`\n  * `c:OpenAPI.Processor.operation_docstring/3`: `OpenAPI.Processor.Operation.docstring/3`\n  * `c:OpenAPI.Processor.operation_request_method/2`: `OpenAPI.Processor.Operation.request_method/2`\n  * `c:OpenAPI.Processor.operation_module_names/2`: `OpenAPI.Processor.Naming.operation_modules/2`\n  * For each operation module name\n    * `c:OpenAPI.Processor.operation_function_name/2`: `OpenAPI.Processor.Naming.operation_function/2`\n    * `c:OpenAPI.Processor.operation_request_body/2`: `OpenAPI.Processor.Operation.request_body/2`\n    * `c:OpenAPI.Processor.operation_response_body/2`: `OpenAPI.Processor.Operation.response_body/2`\n* For each schema\n  * `c:OpenAPI.Processor.ignore_schema?/2`: `OpenAPI.Processor.Ignore.ignore_schema?/2`\n  * `c:OpenAPI.Processor.schema_format/2`: `OpenAPI.Processor.Format.schema_format/2`\n  * `c:OpenAPI.Processor.schema_module_and_type/2`: `OpenAPI.Processor.Naming.schema_module_and_type/2`\n\n**Renderer**: `OpenAPI.Renderer`\n\n* For each file (identified by module name)\n  * `c:OpenAPI.Renderer.render/2`: `OpenAPI.Renderer.Module.render/2`\n    * `c:OpenAPI.Renderer.render_moduledoc/2`: `OpenAPI.Renderer.Module.render_moduledoc/2`\n    * `c:OpenAPI.Renderer.render_using/2`: `OpenAPI.Renderer.Module.render_using/2`\n    * `c:OpenAPI.Renderer.render_default_client/2`: `OpenAPI.Renderer.Module.render_default_client/2`\n    * `c:OpenAPI.Renderer.render_schema/2`: `OpenAPI.Renderer.Schema.render/2`\n      * `c:OpenAPI.Renderer.render_schema_types/2`: `OpenAPI.Renderer.Schema.render_types/2`\n      * `c:OpenAPI.Renderer.render_schema_struct/2`: `OpenAPI.Renderer.Schema.render_struct/2`\n      * `c:OpenAPI.Renderer.render_schema_field_function/2`: `OpenAPI.Renderer.Schema.render_field_function/2`\n    * `c:OpenAPI.Renderer.render_operations/2`: `OpenAPI.Renderer.Operation.render_all/2`\n    * For each operation in the file\n      * `c:OpenAPI.Renderer.render_operation/2`: `OpenAPI.Renderer.Operation.render/2`\n      * `c:OpenAPI.Renderer.render_operation_doc/2`: `OpenAPI.Renderer.Operation.render_doc/2`\n      * `c:OpenAPI.Renderer.render_operation_spec/2`: `OpenAPI.Renderer.Operation.render_spec/2`\n      * `c:OpenAPI.Renderer.render_operation_function/2`: `OpenAPI.Renderer.Operation.render_function/2`\n  * `c:OpenAPI.Renderer.format/2`: `OpenAPI.Renderer.Util.format/2`\n  * `c:OpenAPI.Renderer.location/2`: `OpenAPI.Renderer.Module.filename/2`\n  * `c:OpenAPI.Renderer.write/2`: `OpenAPI.Renderer.Util.write/2`\n\nEach callback represents an opportunity for client library authors to inject their own logic into the code generator.\nIf popular overrides are discovered, they may be added to the default implementation as well.","ref":"plugins.html#callbacks"},{"type":"extras","title":"Client Author Guide","doc":"# Client Author Guide\n\nUnlike most packages on Hex.pm, the target audience for this project is _other library authors_, and not necessarily end-users.\nWhile it is certainly possible for application developers to generate code directly in their Elixir apps, it's likely that most OpenAPI specifications will be consumed by a generic client library (like [oapi_github](https://github.com/aj-foster/open-api-github)) instead.\n\nAs the author of a library with code generated from an OpenAPI description, it's important to establish a healthy relationship between your library and the spec.\nFor example, if your library is based on a specification that is updated often, you likely want to set a reasonable cadence (ex. monthly) for updating the generated code.\nOtherwise, maintenance can become a burden very quickly.\n\nFollowing are some recommendations for how to establish this relationship.","ref":"client-author-guide.html"},{"type":"extras","title":"Cadence - Client Author Guide","doc":"As mentioned above, some OpenAPI descriptions are updated often.\nBefore introducing API versioning, GitHub continuously updated its OpenAPI spec with both new endpoints and corrections for existing ones.\nA library author could potentially release new versions every week in an attempt to keep up.\nHowever, it's unlikely that real-time updates of the generated code is necessary.\n\nHere are a few options for releasing updates to the generated code:\n\n1. If the description updates often with new endpoints that consumers of your library are eager to use, a rapid or as-needed release cadence may be appropriate.\n  This might mean creating a scheduled CI job that regularly re-generates the code based on the latest copy of the specification and opens a pull request with the changes.\n  Or, it might mean encouraging users of the library to open an issue or pull request when they encounter an unsupported endpoint.\n\n2. If the description updates often with fixes to the specification, then a periodic release cadence may be appropriate.\n  This would help users of the library avoid issues while also reducing the maintenance burden.\n\n3. If the description has well-versioned or occasional updates, the client may wish to \"follow\" the release cadence of the specification.\n  In this case, the version of the description may play a central role in the client, and should likely be included as an `.api-version` file in the release.\n  This makes it clear to users of the library what API version they can expect.","ref":"client-author-guide.html#cadence"},{"type":"extras","title":"Contributions - Client Author Guide","doc":"Some libraries will be best served by an as-needed update cadence.\nIn this case, simple guidelines around the contribution of generated code can make a big difference.\n\nHere is an excerpt from the contribution guidelines for the GitHub client library that uses this generator:\n\n>","ref":"client-author-guide.html#contributions"},{"type":"extras","title":"Updating the Generated Code - Client Author Guide","doc":">\n> If you intend to open a PR with updates to the generated code based on the latest GitHub OpenAPI description, please read carefully:\n> \n> 1. Please use the latest commit from the [official repository](https://github.com/github/rest-api-description) at the time of your contribution.\n> 2. Please use `descriptions-next/api.github.com/api.github.com.yaml` — **not** `descriptions` or a GitHub Enterprise release.\n> 3. Use `mix api.gen default path/to/descriptions-next/api.github.com/api.github.com.yaml` to regenerate the code.\n> 4. Please also change the `.api-version` file if appropriate.\n> 5. Please include the commit SHA of the official repository in the description of your pull request.\n> 6. Please do not make any other changes in the same PR (for example, changing this library's version).\n>\n> If you run into any unexpected issues while generating the code, please open an issue.\n>\n> Thank you for your help!\n\nAlthough not all contributors will read this guidance, steps like this can make it much easier for you as a library author to simply merge the pull request and release a new version.","ref":"client-author-guide.html#updating-the-generated-code"},{"type":"extras","title":"Supplemental Specs - Client Author Guide","doc":"Despite best efforts, OpenAPI descriptions often don't cover all of the API endpoints available.\nFor example, although GitHub provides a nearly-comprehensive specification of hundreds of endpoints, there are ID-based endpoints like `/repositories/{id}` that are not publicly documented.\nA library author may know this _a priori_ and wish to include these endpoints in the generated code.\n\nFor this case, it is recommended that the author **maintain a separate, supplemental OpenAPI description document** with just enough information to generate the additional code.\nThis (presumably small) supplemental description can be committed along with the client code while the official description remains separate.\n\n<!-- TODO: Give additional information about referencing existing schemas -->","ref":"client-author-guide.html#supplemental-specs"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}